# -*- coding: utf-8 -*-

from zope.interface import implements

from Products.Archetypes.public import *
# from Products.ATExtensions.ateapi import *

from Products.ATContentTypes.content.folder import ATFolder, ATFolderSchema
from Products.ATContentTypes.content.base import registerATCT
from Products.CMFCore.utils import getToolByName

from persistent.dict import PersistentDict

from jalon.content import contentMessageFactory as _
from jalon.content.config import PROJECTNAME
from jalon.content.interfaces import IJalonBoiteDepot

from DateTime import DateTime
from os import close
from zipfile import ZipFile, ZIP_DEFLATED

import jalon_utils
from jalon_activity import JalonActivity
import os
import copy
import random
import string

# Messages de debug :
from logging import getLogger
LOG = getLogger("[JalonBoiteDepot]")

JalonBoiteDepotSchema = ATFolderSchema.copy() + Schema((
    DateTimeField("dateDepot",
                  required=False,
                  accessor="getDateDepot",
                  searchable=False,
                  widget=CalendarWidget(label=_(u"Date limite de dépôt affichée aux étudiants"),)),
    DateTimeField("dateRetard",
                  required=False,
                  accessor="getDateRetard",
                  searchable=False,
                  widget=CalendarWidget(label=_(u"Tolérer les dépôts jusqu'au"),)),
    LinesField("listeSujets",
               required=False,
               accessor="getListeSujets",
               searchable=False,
               widget=LinesWidget(label=_(u"Liste des sujets"),
                                  visible={'view': 'visible', 'edit': 'invisible'},)),
    LinesField("listeDevoirs",
               required=False,
               accessor="getListeDevoirs",
               searchable=False,
               widget=LinesWidget(label=_(u"Liste des devoirs des étudiants"),
                                  visible={'view': 'visible', 'edit': 'invisible'},)),
    LinesField("listeCorrections",
               required=False,
               accessor="getListeCorrections",
               searchable=False,
               widget=LinesWidget(label=_(u"Liste des corrections"),
                                  visible={'view': 'visible', 'edit': 'invisible'},)),
    StringField("dateAff",
                required=False,
                accessor="getDateAff",
                searchable=False,
                widget=StringWidget(label=_(u"Date d'affichage de la boite de dépôts"),)),
    StringField("dateMasq",
                required=False,
                accessor="getDateMasq",
                searchable=False,
                widget=StringWidget(label=_(u"Date de masquage de la boite de dépôts"),)),
    StringField("profile",
                required=False,
                accessor="getProfile",
                default="standard",
                searchable=False,
                widget=StringWidget(label=_(u"Profil de la boite de dépots"),)),
    BooleanField("correctionIndividuelle",
                 required=False,
                 accessor="getCorrectionIndividuelle",
                 searchable=False,
                 widget=BooleanWidget(label=_(u"Activer les corrections individuelles"),)),
    BooleanField("notificationCorrection",
                 required=False,
                 accessor="getNotificationCorrection",
                 searchable=False,
                 widget=BooleanWidget(label=_(u"Activer la notification des corrections"),)),
    BooleanField("notation",
                 required=False,
                 accessor="getNotation",
                 searchable=False,
                 widget=BooleanWidget(label=_(u"Activer la notation des dépôts des étudiants"),)),
    BooleanField("notificationNotation",
                 required=False,
                 accessor="getNotificationNotation",
                 searchable=False,
                 widget=BooleanWidget(label=_(u"Activer la notification des notations"),)),
    BooleanField("accesDepots",
                 required=False,
                 accessor="getAccesDepots",
                 searchable=False,
                 widget=BooleanWidget(label=_(u"Autoriser la visualisation des dépôts entre étudiants"),)),
    BooleanField("accesCompetences",
                 required=False,
                 accessor="getAccesCompetences",
                 searchable=False,
                 widget=BooleanWidget(label=_(u"Afficher les compétences aux enseignants"),)),
    BooleanField("afficherCompetences",
                 required=False,
                 accessor="getAfficherCompetences",
                 searchable=False,
                 widget=BooleanWidget(label=_(u"Afficher les compétences aux étudiants"),)),
    BooleanField("modifierCompetences",
                 required=False,
                 accessor="getModifierCompetences",
                 searchable=False,
                 widget=BooleanWidget(label=_(u"Restreindre la modification des compétences au créateur de la boite de dépôts"),)),
    DateTimeField("dateCorrection",
                  required=False,
                  accessor="getDateCorrection",
                  searchable=False,
                  widget=CalendarWidget(label=_(u"Date limite de correction"),)),
    IntegerField("nombreCorrection",
                 required=False,
                 accessor="getNombreCorrection",
                 default=2,
                 searchable=False,
                 widget=IntegerWidget(label=_(u"Nombre de correction"),)),
    StringField("penalite",
                required=False,
                accessor="getPenalite",
                default="none",
                searchable=False,
                widget=StringWidget(label=_(u"Pénalité"),)),
    IntegerField("adjustementPoints",
                 required=False,
                 accessor="getAdjustementPoints",
                 default=2,
                 searchable=False,
                 widget=IntegerWidget(label=_(u"Nombre de points d'ajustement"),)),
    BooleanField("accesGrille",
                 required=False,
                 accessor="getAccesGrille",
                 searchable=False,
                 widget=BooleanWidget(label=_(u"Accéder à la grille d'évaluation avant d'évaluer"),)),
    BooleanField("accesEvaluation",
                 required=False,
                 accessor="getAccesEvaluation",
                 default=False,
                 searchable=False,
                 widget=BooleanWidget(label=_(u"Autoriser les étudiants à voir les résultats des évaluations"),)),
    BooleanField("autoriserAutoEvaluation",
                 required=False,
                 accessor="getAutoriserAutoEvaluation",
                 default=False,
                 searchable=False,
                 widget=BooleanWidget(label=_(u"Autoriser les étudiants à réaliser une auto-évaluation"),)),
    BooleanField("affectationEvaluation",
                 required=False,
                 accessor="getAffectationEvaluation",
                 default=False,
                 searchable=False,
                 widget=BooleanWidget(label=_(u"Indique si l'affectation des évaluations a été faite."),)),
))


class JalonBoiteDepot(JalonActivity, ATFolder):
    u"""Une boite de dépôts pour Jalon."""

    implements(IJalonBoiteDepot)
    meta_type = 'JalonBoiteDepot'
    schema = JalonBoiteDepotSchema
    schema['description'].required = False
    schema['description'].widget.label = "Consigne"
    schema['description'].widget.description = ""

    _infos_element = {}
    _competences = {}
    _comp_etudiants = {}
    _crietria_dict = {}
    _peers_dict = {}

    _penality_title = {"none":     "Aucun",
                       "negative": "points en moins si évaluations non réalisées",
                       "positive": "points en plus si évaluations réalisées"}

    _profile_title = {"standard":    "Standard",
                      "examen":      "Examen",
                      "competences": "Évaluation par compétences",
                      "pairs":       "Évaluation par les pairs"}

    def __init__(self, *args, **kwargs):
        super(JalonBoiteDepot, self).__init__(*args, **kwargs)
        super(ATFolder, self).__init__(*args, **kwargs)
        self.setDocumentsProperties({})

    # #-------------------# #
    #  Fonctions générales  #
    # #-------------------# #

    def addMySpaceItem(self, folder_object, item_id, item_type, user_id, display_item, map_position, display_in_plan, portal_workflow):
        """addMySpaceItem."""
        item = super(JalonBoiteDepot, self).addMySpaceItem(folder_object, item_id, item_type, user_id, display_item, map_position, display_in_plan, portal_workflow)
        self.addItemProperty(item["item_id_no_dot"], item["item_type"], item["item_title"], user_id, display_item, item["item_complement"])

    def getDisplayProfile(self, profile_id=None):
        # LOG.info("----- getDisplayProfile -----")
        deposit_box_profil = profile_id or self.getProfile() or "standard"
        return self._profile_title[deposit_box_profil]

    def getDocumentsList(self):
        # LOG.info("----- getDocumentsList -----")
        return self._infos_element.keys()

    def setDocumentsProperties(self, infos_element):
        if type(self._infos_element).__name__ != "PersistentMapping":
            self._infos_element = PersistentDict(infos_element)
        else:
            self._infos_element = infos_element

    def getAttributsMod(self, info):
        dico = {"info":  ['title', 'description'],
                "date":  ['dateDepot', 'dateRetard']}
        return dico[info]

    def getDepositBoxProperty(self, property_name):
        # LOG.info("----- getDepositBoxProperty -----")
        return getattr(self, property_name, None)

    def setProperties(self, dico):
        for key in dico.keys():
            self.__getattribute__("set%s" % key)(dico[key])
        self.reindexObject()

    def getListeAttribut(self, attribut):
        return self.__getattribute__("liste%s" % attribut.capitalize())

    def getInfosListeAttribut(self, attribut, personnel=False):
        retour = []
        listeElement = self.getListeAttribut(attribut)
        infos_element = self.getDocumentsProperties()
        # LOG.info(infos_element)
        for idElement in listeElement:
            infos = infos_element.get(idElement, '')
            if infos:
                affElement = self.isAfficherElement(infos['affElement'], infos['masquerElement'])
                if personnel or not affElement['val'] == 0:
                    new = {"idElement":       idElement,
                           "titreElement":    infos["titreElement"],
                           "typeElement":     infos["typeElement"].replace(" ", ""),
                           "createurElement": infos["createurElement"],
                           "affElement":      affElement,
                           "iconElement":     affElement["icon"],
                           "classElement":    self.test(affElement['val'] == 0, 'arrondi off', 'arrondi')
                           }
                    retour.append(new)
        if retour:
            retour.sort(lambda x, y: cmp(x["titreElement"], y["titreElement"]))
        return retour

    def isPersonnel(self, user, mode_etudiant="false"):
        # LOG.info("----- isPersonnel -----")
        if mode_etudiant == "true":
            # isPersonnel = False (mode étudiant)
            return False
        if user.has_role("Manager"):
            # isPersonnel = True (manager role)
            return True
        if user.has_role("Personnel") and self.isCoAuteurs(user.getId()):
            # isPersonnel = True (Personnel & iscoAuteurs)
            return True
        return False

    def setAttributActivite(self, form):
        if "DateDepot" in form and form["DateDepot"] != self.getDateDepot():
            self.aq_parent.setActuCours({"reference": self.getId(),
                                         "code":      "datedepot",
                                         "dateDepot": form["DateDepot"]})
        self.setProperties(form)
        if "Title" in form:
            self.aq_parent.modifierInfosElementPlan(self.getId(), form["Title"])
        self.reindexObject()

    def getAffDate(self, attribut):
        date = getattr(self, attribut, None)
        if not date:
            return "Aucune date limite de dépôt."
        else:
            return jalon_utils.getLocaleDate(date, '%d %B %Y - %Hh%M')

    def getNbSujets(self, is_personnel=False):
        return len(self.getInfosListeAttribut("sujets", is_personnel))

    def getDepositFileOptions(self):
        options = {}
        for option in ["getCorrectionIndividuelle", "getNotificationCorrection", "getNotation", "getNotificationNotation", "getAccesDepots", "getAccesCompetences"]:
            if self.__getattribute__(option)():
                options[option] = {"actif": 1, "texte": _(u"Activée")}
            else:
                options[option] = {"actif": 0, "texte": _(u"Désactivée")}
        return options

    """
    def retirerTousElements(self):
        dicoRep = {"Image":                     "Fichiers",
                   "File":                      "Fichiers",
                   "Page":                      "Fichiers",
                   "Lien web":                  "Externes",
                   "Lecteur exportable":        "Externes",
                   "Reference bibliographique": "Externes",
                   "Catalogue BU":              "Externes",
                   "Webconference":             "Webconference",
                   "Presentations sonorisees":  "Sonorisation"}
        infos_element = self.getInfosElement()
        for sujet in self.getListeSujets():
            infosElement = infos_element[sujet]
            repertoire = infosElement["typeElement"]
            if repertoire in dicoRep:
                repertoire = dicoRep[repertoire]
            if "*-*" in sujet:
                sujet = sujet.replace("*-*", ".")
            objet = getattr(getattr(getattr(getattr(self.portal_url.getPortalObject(), "Members"), infosElement["createurElement"]), repertoire), sujet, None)
            if objet:
                relatedItems = objet.getRelatedItems()
                try:
                    relatedItems.remove(self)
                    objet.setRelatedItems(relatedItems)
                except:
                    pass
                objet.reindexObject()
    """

    def getIconClass(self):
        """Return the Activity icon CSS class."""
        # LOG.info("----- getIconClass -----")
        return "fa fa-inbox"

    def getDepositBoxProfile(self):
        # LOG.info("----- getDepositBoxProfile -----")
        deposit_box_profil = self.getProfile() or "standard"
        # LOG.info("***** deposit_box_profil : %s" % deposit_box_profil)
        return [{"deposit_box_profile_text":    "Standard",
                 "deposit_box_profile_value":   "standard",
                 "deposit_box_profile_checked": "selected" if deposit_box_profil == "standard" else "",
                 "deposit_box_profile_help":    "Les étudiants peuvent avoir autant de fichiers actifs qu'ils le désirent, les télécharger et les invalider. La date limite de dépôts est facultative."},
                {"deposit_box_profile_text":    "Examen",
                 "deposit_box_profile_value":   "examen",
                 "deposit_box_profile_checked": "selected" if deposit_box_profil == "examen" else "",
                 "deposit_box_profile_help":    "Les étudiants ne peuvent avoir qu'un seul fichier actif. Ils ne peuvent pas télécharger les fichiers qu'ils ont déposé. La date limite de dépôts est facultative."},
                {"deposit_box_profile_text":    "Évaluation par compétences",
                 "deposit_box_profile_value":   "competences",
                 "deposit_box_profile_checked": "selected" if deposit_box_profil == "competences" else "",
                 "deposit_box_profile_help":    "Les étudiants ne peuvent avoir qu'un seul fichier actif. La date limite de dépôts est facultative. Vous pouvez créer et évaluer vos étudiants à l'aide d'une grille de compétences accessible depuis l'onglet Compétences."},
                {"deposit_box_profile_text":    "Évaluation par les pairs",
                 "deposit_box_profile_value":   "pairs",
                 "deposit_box_profile_checked": "selected" if deposit_box_profil == "pairs" else "",
                 "deposit_box_profile_help":    "Les étudiants ne peuvent avoir qu'un seul fichier actif. À partir de l'onglet Par les pairs, vous pourrez créer une grille d'évaluations et demander à vos étudiants d'évaluer leurs pairs. Les dates limites de dépots et d'évaluation sont obligatoires."}]

    def isExamen(self):
        # LOG.info("----- isExamen -----")
        return True if self.getProfile() == "examen" else False

    def isNotStandard(self):
        # LOG.info("----- isNotStandard -----")
        return True if self.getProfile() != "standard" else False

    # #-----------------------# #
    #  Fonctions onglet Dépots  #
    # #-----------------------# #

    def addDepositFile(self, deposit_title, desposit_comment, deposit_file, user_id):
        # LOG.info("----- addDepositFile -----")
        if self.isNotStandard():
            content_filter = {"portal_type": "JalonFile", "Creator": user_id}
            depots = self.getFolderContents(contentFilter=content_filter)
            for depot_brain in depots:
                depot = depot_brain.getObject()
                # LOG.info("***** object_id : %s" % depot.getId())
                depot.setProperties({"Actif": ""})

        part1 = ''.join([random.choice(string.ascii_lowercase) for i in range(3)])
        part2 = ''.join([random.choice(string.digits[1:]) for i in range(3)])
        file_id = "Depot-%s%s-%s" % (part1, part2, DateTime().strftime("%Y%m%d%H%M%S"))
        self.invokeFactory(type_name='JalonFile', id=file_id)

        deposit_object = getattr(self, file_id)
        deposit_object.setProperties({"Title":       deposit_title,
                                      "Description": desposit_comment,
                                      "File":        deposit_file})
        self.aq_parent.setActuCours({"reference": self.getId(),
                                     "code":      "nouveauxdepots"})

        comp_etudiants = dict(self.getCompEtudiants())
        listeEtu = comp_etudiants.keys()
        if not user_id in listeEtu:
            comp_etudiants[user_id] = {}
            self.setCompEtudiants(comp_etudiants)

        peers_dict = dict(self.getPeersDict())
        peers_list = peers_dict.keys()
        if not user_id in peers_list:
            peers_dict[user_id] = []
            self.setPeersDict(peers_dict)

    def getDepots(self, auth_member, is_personnel, is_depot_actif, is_retard):
        valides = 0
        liste_depots = []
        liste_etudiants = []
        liste_etudiants_valides = []
        dico_name_etudiants = {}
        auth_member_id = auth_member.getId()
        is_not_standard = self.isNotStandard()

        menus = []
        if is_personnel:
            menus.append({"href": "%s/download_deposit_zip_form" % self.absolute_url(),
                          "icon": "fa-file-archive-o",
                          "text": "Télécharger les dépôts (ZIP)"})
            menus.append({"href": "%s/download_deposit_list_form" % self.absolute_url(),
                          "icon": "fa-list",
                          "text": "Télécharger listing"})
            menus.append({"href": "%s/delete_deposit_files_form" % self.absolute_url(),
                          "icon": "fa-trash-o alert",
                          "text": "Supprimer les dépôts"})

        table_title = "Dépôts étudiants"
        content_filter = {"portal_type": "JalonFile"}
        acces_depots = self.getAccesDepots()
        if not is_personnel and not acces_depots:
            table_title = "Mes dépôts"
            content_filter["Creator"] = auth_member_id
        depots = self.getFolderContents(contentFilter=content_filter)

        if not depots:
            return {"table_title":     table_title,
                    "is_infos_depots": False,
                    "infos_depots":    "",
                    "liste_depots":    [],
                    "acces_depots":    acces_depots,
                    "menus":           menus}

        is_corriger_noter = self.isCorrigerNoter()
        for depot in depots:
            # créateur du dépôt = etudiant
            depot_id = depot.getId
            etudiant_id = str(depot.Creator)
            if not etudiant_id in liste_etudiants:
                liste_etudiants.append(etudiant_id)
                etudiant_name = self.getIndividu(etudiant_id, "dict")["fullname"]
                #if etudiant_infos:
                #    etudiant_name = "%s %s" % (etudiant_infos["nom"], etudiant_infos["prenom"])
                #else:
                #    etudiant_name = etudiant_id
                dico_name_etudiants[etudiant_id] = etudiant_name
            else:
                etudiant_name = dico_name_etudiants[etudiant_id]

            columns_etat_correction_notation = []

            is_valide = {"value": False, "test": "is_column_etat", "css_class": "valide", "span_css_class": "label warning", "text": "Invalide"}
            if depot.getActif:
                # LOG.info("getActif : %s" % depot.getActif)
                is_valide = {"value": True, "test": "is_column_etat", "css_class": "valide", "span_css_class": "label success", "text": "Valide"}
                valides = valides + 1
                if not etudiant_id in liste_etudiants_valides:
                    liste_etudiants_valides.append(etudiant_id)
            columns_etat_correction_notation.append(is_valide)

            is_correction = {"value": False, "test": "is_column_correction", "css_class": "correction", "span_css_class": "label secondary", "text": "Non corrigé"}
            if depot.getCorrection not in ["", None, " "]:
                is_correction = {"value": True, "test": "is_column_correction", "css_class": "correction", "span_css_class": "label success", "text": "Corrigé"}
            if depot.getFichierCorrection:
                is_correction = {"value": True, "test": "is_column_correction", "css_class": "correction", "span_css_class": "label success", "text": "Corrigé"}
            columns_etat_correction_notation.append(is_correction)

            note = depot.getNote
            is_notation = {"value": False, "test": "is_column_notation", "css_class": "note", "span_css_class": "label secondary", "text": "Non noté"}
            if note:
                is_notation = {"value": True, "test": "is_column_notation", "css_class": "note", "span_css_class": "", "text": note}
            columns_etat_correction_notation.append(is_notation)

            is_corrupt = False
            depot_action = {"action_title": "Consulter",
                            "action_url":   "%s/%s" % (self.absolute_url(), depot_id),
                            "action_icon":  "fa-eye",
                            "action_text":  "Consulter"}
            if etudiant_id == auth_member_id:
                if depot.getObject().getSize() < 100:
                    is_corrupt = True
                if not is_not_standard:
                    if not (is_correction["value"] or is_notation["value"]):
                        depot_action = {"action_title": "Modifier l'état ce dépôt",
                                        "action_url":   "%s/edit_validate_deposit_form" % depot.getURL(),
                                        "action_icon":  "fa-pencil",
                                        "action_text":  "Modifier"}
            if is_personnel and is_corriger_noter:
                depot_action = {"action_title": is_corriger_noter["title"],
                                "action_url":   "%s/%s/correct_and_evaluate_deposit_file_form" % (self.absolute_url(), depot_id),
                                "action_icon":  "fa-legal",
                                "action_text":  is_corriger_noter["title"]}

            creation_date = self.getLocaleDate(depot.created, '%d/%m/%Y - %Hh%M')

            liste_depots.append({"etudiant_name":                    etudiant_name,
                                 "depot_id":                         depot_id,
                                 "depot_title":                      depot.Title,
                                 "depot_date":                       creation_date,
                                 "depot_url":                        depot.getURL(),
                                 "date_sort":                        self.getDepotDate(creation_date, 1),
                                 "date_aff":                         self.getDepotDate(creation_date),
                                 "is_corrupt":                       is_corrupt,
                                 "columns_etat_correction_notation": columns_etat_correction_notation,
                                 "depot_action":                     depot_action})

        infos_depots = {}
        is_infos_depots = False
        if is_personnel:
            if len(liste_depots):
                is_infos_depots = True
                infos_depots["css_class"] = "callout"
                text_infos_depots = ["Il y a actuellement"]
                if valides == 1:
                    text_infos_depots.append("<strong>%s</strong> dépôt valide envoyé par" % valides)
                else:
                    text_infos_depots.append("<strong>%s</strong> dépôts valides envoyés par" % valides)
                taille_liste_etudiants_valides = len(liste_etudiants_valides)
                if taille_liste_etudiants_valides == 1:
                    text_infos_depots.append("<strong>%s</strong> étudiant." % taille_liste_etudiants_valides)
                else:
                    text_infos_depots.append("<strong>%s</strong> étudiant." % taille_liste_etudiants_valides)
                infos_depots["text"] = " ".join(text_infos_depots)
        else:
            if not is_depot_actif:
                is_infos_depots = True
                infos_depots = {"css_class": "warning",
                                "text":      "<i class=\"fa fa-warning\"></i>La date limite autorisée a été dépassée. Vous ne pouvez plus déposer."}
            if is_retard:
                is_infos_depots = True
                infos_depots = {"css_class": "warning",
                                "text":      "<i class=\"fa fa-warning\"></i>Vous êtes en retard. Dernières minutes avant la fermeture des dépôts."}

        retour = {"table_title":     table_title,
                  "is_infos_depots": is_infos_depots,
                  "infos_depots":    infos_depots,
                  "liste_depots":    liste_depots,
                  "acces_depots":    acces_depots,
                  "menus":           menus}
        return retour

    def getInfosTableau(self, is_personnel, is_depot_actif):
        head_table = []
        is_column_correction = self.getCorrectionIndividuelle()
        is_column_notation = self.getNotation()
        #is_personnel_and_is_actions = True if is_personnel and (is_colmun_correction or is_colmun_notation) else False
        is_colmun_etudiant = True if is_personnel or self.getAccesDepots() else False
        if is_colmun_etudiant:
            head_table.append({"css_class":    "sort text-left has-tip",
                               "attr_title":   "Trier selon l'étudiant",
                               "data-sort":    "name",
                               "column_title": "Étudiant"})
        head_table.append({"css_class":    "sort text-left has-tip",
                           "attr_title":   "Trier selon la date",
                           "data-sort":    "title",
                           "column_title": "Dépôt"})
        head_table.append({"css_class":    "sort text-left has-tip",
                           "attr_title":   "Trier selon l'état",
                           "data-sort":    "valide",
                           "column_title": "État"})
        if is_column_correction:
            head_table.append({"css_class":    "sort text-left has-tip",
                               "attr_title":   "Trier selon la correction",
                               "data-sort":    "correction",
                               "column_title": "Correction"})
        if is_column_notation:
            head_table.append({"css_class":    "sort text-left has-tip",
                               "attr_title":   "Trier selon la note",
                               "data-sort":    "note",
                               "column_title": "Note"})
        #if is_personnel_and_is_actions:
        #    head_table.append({"css_class":    "nosort",
        #                       "attr_title":   "",
        #                       "data-sort":    "",
        #                       "column_title": "<i class=\"fa fa-cog\"></i>"})

        is_column_actions = False
        if is_personnel:
            if is_column_correction or is_column_notation:
                is_column_actions = True
        else:
            if is_depot_actif:
                is_column_actions = True
            if is_column_correction:
                is_column_actions = True
        if is_column_actions:
            head_table.append({"css_class":    "nosort",
                               "attr_title":   "",
                               "data-sort":    "",
                               "column_title": "<i class=\"fa fa-cog\"></i>"})
        return {"head_table":           head_table,
                "is_column_etudiant":   is_colmun_etudiant,
                "is_column_etat":       True,
                "is_column_correction": is_column_correction,
                "is_column_notation":   is_column_notation,
                "is_column_actions":    is_column_actions,
                "option":               self.getDepositFileOptions()}
        #        "is_personnel_and_is_actions": is_personnel_and_is_actions,

    def getDepotDate(self, data, sortable=False):
        return jalon_utils.getDepotDate(data, sortable)

    def getDateDepot(self):
        if not self.dateDepot:
            return DateTime().strftime("%Y/%m/%d %H:%M")
        else:
            return self.dateDepot.strftime("%Y/%m/%d %H:%M")

    def getDateRetard(self):
        if not self.dateRetard:
            return DateTime().strftime("%Y/%m/%d %H:%M")
        else:
            return self.dateRetard.strftime("%Y/%m/%d %H:%M")

    def getDateCorrection(self):
        try:
            return self.dateCorrection.strftime("%Y/%m/%d %H:%M")
        except:
            return DateTime().strftime("%Y/%m/%d %H:%M")
        #if not self.dateCorrection:
        #    return DateTime().strftime("%Y/%m/%d %H:%M")
        #else:
        #    return self.dateCorrection.strftime("%Y/%m/%d %H:%M")

    def isFinishDeposit(self):
        # LOG.info("----- isFinishDeposit -----")
        now = DateTime(DateTime()).strftime("%Y/%m/%d %H:%M")
        return True if now > self.getDateDepot() else False

    def isFinishEvaluation(self):
        # LOG.info("----- isFinishEvaluation -----")
        now = DateTime(DateTime()).strftime("%Y/%m/%d %H:%M")
        return True if now > self.getDateCorrection() else False

    def isPossibleAffection(self):
        # LOG.info("----- isPossibleAffection -----")
        return True if (not self.getAffectationEvaluation()) and self.getPeerLength(True, "") else False

    def getNbDepots(self, is_personnel, user_id):
        # LOG.info("----- getNbDepots -----")
        if not is_personnel and not self.getAccesDepots():
            # LOG.info("***** 1")
            #nbDepots = 0
            #authMember = self.portal_membership.getAuthenticatedMember().getId()
            #for iddepot in self.objectIds():
            #    if authMember in iddepot:
            #        nbDepots = nbDepots + 1
            #return nbDepots
            return len(self.getFolderContents(contentFilter={"portal_type": "JalonFile", "Creator": user_id}))
        return len(self.getFolderContents(contentFilter={"portal_type": "JalonFile"}))
        #depots = self.objectIds()
        #if "corrections" in depots:
        #    # LOG.info("***** 2")
        #    return len(depots) - 1
        #else:
        #    # LOG.info("***** 3")
        #    # LOG.info(depots)
        #    return len(depots)

    """
    isDepotActif renvoit :
        1 si les étudiants ont le droit de déposer.
        2 s'ils sont en retard.
        0 s'ils n'ont plus le droit.
    """
    def isDepotActif(self):
        # LOG.info("----- isDepotActif -----")
        profile = self.getProfile()
        now = DateTime(DateTime()).strftime("%Y/%m/%d %H:%M")
        date_depot = DateTime(self.getDateDepot()).strftime("%Y/%m/%d %H:%M")

        # LOG.info("***** profile : %s" % profile)
        # LOG.info("***** now : %s" % now)
        # LOG.info("***** date_depot : %s" % date_depot)
        # En profil évaluation par les pairs la date de dépôts est obligatoire
        if profile == "pairs" and date_depot == now:
            return 3

        date_correction = DateTime(self.getDateCorrection()).strftime("%Y/%m/%d %H:%M")
        # LOG.info("***** date_correction : %s" % date_correction)
        # En profil évaluation par les pairs la date de correction est obligatoire
        if profile == "pairs" and date_correction == now:
            return 3

        # La date de dépot n'est pas encore passée.
        if now <= date_depot:
            return 1

        date_retard = DateTime(self.getDateRetard()).strftime("%Y/%m/%d %H:%M")
        # La date de retard n'est pas encore passée.
        if date_retard > date_depot and now < date_retard:
            return 2
        return 0

    def isCorrigerNoter(self):
        corriger = self.getCorrectionIndividuelle()
        noter = self.getNotation()
        if corriger and noter:
            return {"title":   "Corriger et Noter",
                    "corriger": 1,
                    "noter":    1}
        if corriger:
            return {"title":   "Corriger",
                    "corriger": 1,
                    "noter":    0}
        if noter:
            return {"title":    "Noter",
                    "corriger": 0,
                    "noter":    1}

    def activerDepot(self, idDepot, actif):
        depot = getattr(self, idDepot)
        depot.setProperties({"Actif": actif})

    def purgerDepots(self):
        object_ids = [obj_brain.getId for obj_brain in self.getFolderContents(contentFilter={"portal_type": "JalonFile"})]
        self.manage_delObjects(object_ids)
        self.setListeDevoirs(())
        self.setCompEtudiants({})
        self.setPeersDict({})
        self.setProperties({"AffectationEvaluation": False})
        portal_jalon_bdd = getToolByName(self, "portal_jalon_bdd")
        portal_jalon_bdd.deletePeersEvaluation(self.getId())
        self.reindexObject()

    def telechargerDepots(self, HTTP_USER_AGENT):
        LOG.info("----- telechargerDepots START -----")
        import tempfile
        fd, path = tempfile.mkstemp('.zipfiletransport')
        close(fd)

        zipFile = ZipFile(path, 'w', ZIP_DEFLATED)

        #dicoEtu = {}
        listeDepots = []
        listeEtudiants = []

        for obj_brain in self.getFolderContents(contentFilter={"portal_type": "JalonFile"}):
            obj = obj_brain.getObject()
            if obj.getActif() == "actif":
                idEtudiant = obj.Creator()
                if not idEtudiant in listeEtudiants:
                    listeEtudiants.append(idEtudiant)
                listeDepots.append({"idEtudiant": idEtudiant,
                                    "filename":   "(%s) %s" % (DateTime(obj.created()).strftime("%Y-%m-%d %Hh%Mm%Ss"), obj.file.filename),
                                    "file_data":  str(obj.file.data)})

        dicoEtudiants = jalon_utils.getIndividus(listeEtudiants, type="dict")
        LOG.info("dicoEtudiants")
        LOG.info(dicoEtudiants)
        for depot in listeDepots:
            try:
                filename_path = "%s/%s %s (%s)/%s" % (self.Title(), dicoEtudiants[depot["idEtudiant"]]["nom"].encode("utf-8"), dicoEtudiants[depot["idEtudiant"]]["prenom"].encode("utf-8"), dicoEtudiants[depot["idEtudiant"]]["num_etu"].encode("utf-8"), depot["filename"])
            except:
                filename_path = "%s/%s/%s" % (self.Title(), depot["idEtudiant"], depot["filename"])
            if 'Windows' in HTTP_USER_AGENT:
                try:
                    filename_path = filename_path.decode('utf-8').encode('cp437')
                except:
                    pass
            zipFile.writestr(filename_path, depot["file_data"])
        zipFile.close()

        fp = open(path, 'rb')
        data = fp.read()
        fp.close()
        LOG.info("----- telechargerDepots END -----")
        return {"length": str(os.stat(path)[6]), "data": data}

    def telechargerListingDepots(self, HTTP_USER_AGENT):
        # LOG.info("----- telechargerListingDepots -----")
        import tempfile
        from xlwt import Workbook

        isCorrection = self.getCorrectionIndividuelle()
        isNotation = self.getNotation()
        translation_service = getToolByName(self, 'translation_service')
        portal_membership = getToolByName(self, "portal_membership")
        authMember = portal_membership.getAuthenticatedMember()

        fd, path = tempfile.mkstemp('.%s-xlfiletransport' % authMember.getId())
        close(fd)

        # création
        listing = Workbook(encoding="utf-8")

        # création de la feuille 1
        feuil1 = listing.add_sheet('feuille 1')

        # ajout des en-têtes
        feuil1.write(0, 0, 'Nom')
        feuil1.write(0, 1, 'Prénom')
        feuil1.write(0, 2, 'Numéro étudiant')
        feuil1.write(0, 3, 'Titre du dépôt')
        feuil1.write(0, 4, 'Date du dépôt')
        if isCorrection:
            feuil1.write(0, 5, 'Correction')
        if isNotation:
            feuil1.write(0, 6, 'Note')

        listeDepots = []
        listeEtudiants = []

        for obj in self.getFolderContents(contentFilter={"portal_type": "JalonFile"}):
            # LOG.info("JalonFile")
            if obj.getActif == "actif":
                # LOG.info("actif")
                idEtudiant = obj.Creator
                depot = {"idEtudiant": idEtudiant,
                         "titreDepot": obj.Title,
                         "dateDepot":  self.getLocaleDate(obj.created, '%d/%m/%Y - %Hh%M')}
                if isCorrection:
                    correction = obj.getCorrection
                    if not correction:
                        dummy = _(u"Non corrigé")
                        msg_correction = u"Non corrigé"
                        correction = translation_service.utranslate(domain='jalon.content',
                                                                    msgid=msg_correction,
                                                                    default=msg_correction,
                                                                    context=object)
                    depot["correctionDepot"] = correction
                if isNotation:
                    note = obj.getNote
                    if not note:
                        dummy = _(u"Non noté")
                        msg_note = u"Non noté"
                        note = translation_service.utranslate(domain='jalon.content',
                                                              msgid=msg_note,
                                                              default=msg_note,
                                                              context=object)
                    depot["noteDepot"] = note
                listeDepots.append(depot)
                if not idEtudiant in listeEtudiants:
                    listeEtudiants.append(idEtudiant)
        # LOG.info(listeEtudiants)
        dicoEtudiants = jalon_utils.getIndividus(listeEtudiants, type="dict")

        i = 1
        for depot in listeDepots:
            if depot["idEtudiant"] in dicoEtudiants.keys():
                # ajout des valeurs dans la ligne suivante
                ligne1 = feuil1.row(i)
                ligne1.write(0, dicoEtudiants[depot["idEtudiant"]]["nom"])
                ligne1.write(1, dicoEtudiants[depot["idEtudiant"]]["prenom"])
                ligne1.write(2, dicoEtudiants[depot["idEtudiant"]]["num_etu"])
                ligne1.write(3, depot["titreDepot"])
                ligne1.write(4, depot["dateDepot"])
                if isCorrection:
                    ligne1.write(5, depot["correctionDepot"])
                if isNotation:
                    ligne1.write(6, depot["noteDepot"])
                i = i + 1

        # ajustement éventuel de la largeur d'une colonne
        feuil1.col(2).width = 4000
        feuil1.col(3).width = 10000
        feuil1.col(4).width = 4000
        if isCorrection:
            feuil1.col(5).width = 10000

        listing.save(path)

        fp = open(path, 'rb')
        data = fp.read()
        fp.close()
        return {"length": str(os.stat(path)[6]), "data": data}

    # #----------------------------# #
    #  Fonctions onglet Compétences  #
    # #----------------------------# #
    def getOrdreCompetences(self):
        ordre = self._competences.keys()
        ordre.sort(lambda x, y: cmp(int(x), int(y)))
        return ordre

    def getCompetences(self, key=None):
        if key:
            return self._competences.get(key, None)
        return self._competences

    def setCompetences(self, competences):
        if type(self._competences).__name__ != "PersistentMapping":
            self._competences = PersistentDict(competences)
        else:
            self._competences = competences

    def getOrdreEtudiants(self):
        ordre = []
        for SESAME_ETU in self._comp_etudiants.keys():
            ordre.append({"id":  SESAME_ETU,
                          "nom": self.getNomEtudiant(SESAME_ETU)})
        ordre.sort(lambda x, y: cmp(x["nom"], y["nom"]))
        return ordre

    def getCompEtudiants(self, key=None):
        if key:
            return self._comp_etudiants.get(key, None)
        return self._comp_etudiants

    def getAffComEtu(self, SESAME_ETU):
        dicoCompEtudiants = self._comp_etudiants
        if not SESAME_ETU in dicoCompEtudiants:
            return {}

        dicoCompEtudiant = {}
        compEtu = dicoCompEtudiants[SESAME_ETU]
        dicoGradation = {"acquerir" : "À acquérir",
                         "encours"  : "En cours d'acquisition",
                         "partielle": "Partiellement acquis",
                         "acquise"  : "Acquis"}
        dicoCompetences = self._competences
        listeIdCompetences = self._competences.keys()
        listeIdCompetences.sort(lambda x, y: cmp(int(x), int(y)))
        for idcompetence in listeIdCompetences:
            competence = dicoCompetences[idcompetence]
            if idcompetence in compEtu:
                if competence["evaluation"] == "libre":
                    dicoCompEtudiant[idcompetence] = compEtu[idcompetence]
                if competence["evaluation"] == "validation":
                    dicoCompEtudiant[idcompetence] = "Acquis"
                if competence["evaluation"] == "gradation":
                    dicoCompEtudiant[idcompetence] = dicoGradation[compEtu[idcompetence]]
                if competence["evaluation"] == "note":
                    if "." in compEtu[idcompetence] or "," in compEtu[idcompetence]:
                        if competence["note_partielle"]:
                            note_partielle = float(competence["note_partielle"])
                        if competence["note_acquise"]:
                            note_acquise = float(competence["note_acquise"])
                        note = float(compEtu[idcompetence])
                    else:
                        if competence["note_partielle"]:
                            note_partielle = int(competence["note_partielle"])
                        if competence["note_acquise"]:
                            note_acquise = int(competence["note_acquise"])
                        try:
                            note = int(compEtu[idcompetence])
                        except:
                            note = "Pas de note"
                    if note_acquise and note >= note_acquise:
                        dicoCompEtudiant[idcompetence] = "Acquis"
                        #dicoCompEtudiant[idcompetence] = "%s / %s (Acquis)" % (compEtu[idcompetence], competence["note_max"])
                    elif note_partielle and note >= note_partielle:
                        dicoCompEtudiant[idcompetence] = "Partiellement acquis"
                        #dicoCompEtudiant[idcompetence] = "%s / %s (Partiellement acquis)" % (compEtu[idcompetence], competence["note_max"])
                    else:
                        dicoCompEtudiant[idcompetence] = "À acquérir"
                        #dicoCompEtudiant[idcompetence] = compEtu[idcompetence]
            else:
                dicoCompEtudiant[idcompetence] = 'À acquérir'
        return dicoCompEtudiant

    def getNomEtudiant(self, SESAME_ETU):
        if self.isLDAP() or "@" in SESAME_ETU:
            #Cas du ldap actif ou de l'etudiant EtudiantJalon
            portal_membership = getToolByName(self, "portal_membership")
            member = portal_membership.getMemberById(SESAME_ETU)
            if member:
                etudiantName = str(member.getProperty("fullname"))
                if not etudiantName:
                    etudiantName = str(member.getProperty("cn"))
            else:
                etudiantName = SESAME_ETU
        else:
            portal_jalon_bdd = getToolByName(self, "portal_jalon_bdd")
            etudiant = portal_jalon_bdd.getIndividuLITE(SESAME_ETU)
            if etudiant:
                etudiantName = "%s %s" % (etudiant["LIB_NOM_PAT_IND"], etudiant["LIB_PR1_IND"])
            else:
                etudiantName = SESAME_ETU
        return etudiantName

    def setCompEtudiants(self, comp_etudiants):
        if type(self._comp_etudiants).__name__ != "PersistentMapping":
            self._comp_etudiants = PersistentDict(comp_etudiants)
        else:
            self._comp_etudiants = comp_etudiants

    def getNbCompetences(self):
        return len(self._competences.keys())

    def getPermissionModifierCompetence(self, personnel, user_id):
        if not personnel:
            return False
        elif user_id == self.Creator():
            return True
        elif self.getModifierCompetences():
            return False
        else:
            return True

    def telechargerListingCompetences(self):
        import tempfile
        from xlwt import Workbook, XFStyle, Style, Pattern, Font

        dicoGradation = {"acquerir":  "À acquérir",
                         "encours":   "En cours d'acquisition",
                         "partielle": "Partiellement acquis",
                         "acquise":   "Acquis"}
        portal_membership = getToolByName(self, "portal_membership")
        authMember = portal_membership.getAuthenticatedMember()

        fd, path = tempfile.mkstemp('.%s-xlfiletransport' % authMember.getId())
        close(fd)

        # création
        listing = Workbook(encoding="utf-8")

        # création de la feuille 1
        try:
            feuil1 = listing.add_sheet(self.Title())
        except:
            feuil1 = listing.add_sheet("Boite de dépôts Jalon")

        styleAcquerir = XFStyle()
        patternAcquerir = Pattern()
        patternAcquerir.pattern = Pattern.SOLID_PATTERN
        patternAcquerir.pattern_fore_colour = Style.colour_map['gray25']
        styleAcquerir.pattern = patternAcquerir

        styleEnCours = XFStyle()
        patternEnCours = Pattern()
        patternEnCours.pattern = Pattern.SOLID_PATTERN
        patternEnCours.pattern_fore_colour = Style.colour_map['light_green']
        styleEnCours.pattern = patternEnCours

        styleAcquise = XFStyle()
        patternAcquise = Pattern()
        patternAcquise.pattern = Pattern.SOLID_PATTERN
        patternAcquise.pattern_fore_colour = Style.colour_map['sea_green']
        styleAcquise.pattern = patternAcquise
        styleAcquise.font.colour_index = Style.colour_map['white']

        bold = XFStyle()
        font = Font()
        font.bold = True
        bold.font = font

        feuil1.write(0, 0, "Légende", bold)
        feuil1.write(0, 1, "À acquérir", styleAcquerir)
        feuil1.write(0, 2, "En cours d'acquisition", styleEnCours)
        feuil1.write(0, 3, "Acquis", styleAcquise)

        styleEnTete = XFStyle()
        patternEnTete = Pattern()
        patternEnTete.pattern = Pattern.SOLID_PATTERN
        patternEnTete.pattern_fore_colour = Style.colour_map["black"]
        styleEnTete.pattern = patternEnTete
        styleEnTete.font.colour_index = Style.colour_map["white"]

        # ajout des en-têtes
        feuil1.write(2, 0, "Nom", styleEnTete)
        feuil1.write(2, 1, "Prénom", styleEnTete)
        feuil1.write(2, 2, "Numéro étudiant", styleEnTete)

        feuil1.col(0).width = 9 * 256
        feuil1.col(1).width = 12 * 256
        feuil1.col(2).width = 22 * 256

        listeIdCompetences = self._competences.keys()
        listeIdCompetences.sort(lambda x, y: cmp(int(x), int(y)))

        i = 3
        dicoCompetences = self._competences
        for idcompetence in listeIdCompetences:
            feuil1.write(2, i, dicoCompetences[idcompetence]["titre"], styleEnTete)
            feuil1.col(i).width = (len(dicoCompetences[idcompetence]["titre"]) + 2) * 256
            i = i + 1

        listeEtudiants = self._comp_etudiants.keys()
        dicoEtudiants = jalon_utils.getIndividus(listeEtudiants, type="dict")
        dicoCompEtudiant = self._comp_etudiants

        i = 3
        for etudiant in listeEtudiants:
            if etudiant in dicoEtudiants.keys():
                # ajout des valeurs dans la ligne suivante
                ligne1 = feuil1.row(i)
                ligne1.write(0, dicoEtudiants[etudiant]["nom"])
                ligne1.write(1, dicoEtudiants[etudiant]["prenom"])
                ligne1.write(2, dicoEtudiants[etudiant]["num_etu"])

                index_comp = 3
                compEtu = dicoCompEtudiant[etudiant]
                for idcompetence in listeIdCompetences:
                    competence = dicoCompetences[idcompetence]
                    if idcompetence in compEtu:
                        if competence["evaluation"] == "libre":
                            ligne1.write(index_comp, compEtu[idcompetence])
                        if competence["evaluation"] == "validation":
                            ligne1.write(index_comp, "Acquis", styleAcquise)
                        if competence["evaluation"] == "gradation":
                            if compEtu[idcompetence] == "acquerir":
                                ligne1.write(index_comp, dicoGradation[compEtu[idcompetence]], styleAcquerir)
                            if compEtu[idcompetence] == "encours":
                                ligne1.write(index_comp, dicoGradation[compEtu[idcompetence]], styleEnCours)
                            if compEtu[idcompetence] == "acquise":
                                ligne1.write(index_comp, dicoGradation[compEtu[idcompetence]], styleAcquise)
                        if competence["evaluation"] == "note":
                            style = XFStyle()
                            if "." in compEtu[idcompetence] or "," in compEtu[idcompetence]:
                                style.num_format_str = "0.00"
                                if competence["note_partielle"]:
                                    note_partielle = float(competence["note_partielle"])
                                if competence["note_acquise"]:
                                    note_acquise = float(competence["note_acquise"])
                                note = float(compEtu[idcompetence])
                            else:
                                style.num_format_str = "0"
                                if competence["note_partielle"]:
                                    note_partielle = int(competence["note_partielle"])
                                if competence["note_acquise"]:
                                    note_acquise = int(competence["note_acquise"])
                                note = int(compEtu[idcompetence])
                            if note_acquise and note >= note_acquise:
                                style.pattern = patternAcquise
                            elif note_partielle and note >= note_partielle:
                                style.pattern = patternEnCours
                            elif note_acquise and note_partielle:
                                style.pattern = patternAcquerir

                            ligne1.write(index_comp, note, style)
                    else:
                        ligne1.write(index_comp, 'À acquérir', styleAcquerir)
                    index_comp = index_comp + 1
            i = i + 1

        listing.save(path)

        fp = open(path, 'rb')
        data = fp.read()
        fp.close()
        return {"length": str(os.stat(path)[6]), "data": data}

    # #------------------------# #
    #  Évaluation par les pairs  #
    # #------------------------# #
    def isEvaluationByPeers(self):
        # LOG.info("----- isEvaluationByPeers -----")
        return True if self.getProfile() == "pairs" else False

    def getAccesGrille(self, is_personnel=False):
        # LOG.info("----- getAccesGrille -----")
        try:
            return True if is_personnel else self.accesGrille
        except:
            return True if is_personnel else False

    def getEvaluationByPeers(self, user, is_personnel):
        # LOG.info("----- getEvaluationByPeers -----")
        evaluation_by_peers_dict = {}
        deposit_box_link = self.absolute_url()
        evaluation_by_peers_dict["is_evaluable"] = self.isEvaluable()
        evaluation_by_peers_dict["criteria_dict"] = self.getCriteriaDict()
        evaluation_by_peers_dict["criteria_order"] = self.getCriteriaOrder()
        if is_personnel:
            evaluation_by_peers_dict["macro_peers"] = "peers_teacher_macro"
            evaluation_by_peers_dict["table_title"] = "Évaluation par les pairs"

            evaluation_by_peers_dict["options"] = [{"text": "Affecter",
                                                    "link": "%s/affect_deposit_file_form" % deposit_box_link,
                                                    "icon": "fa fa-rss fa-fw"},
                                                   {"text": "Moyenne",
                                                    "link": "%s/average_deposit_file_form" % deposit_box_link,
                                                    "icon": "fa fa-calculator fa-fw"},
                                                   {"text": "Résultats",
                                                    "link": "%s/edit_peers_results_form" % deposit_box_link,
                                                    "icon": "fa-trophy"},
                                                   {"text": "Télécharger les notes",
                                                    "link": "%s/download_peers_results_page" % deposit_box_link,
                                                    "icon": "fa-list"}]

            #if not evaluation_by_peers_dict["criteria_dict"]:
            #    evaluation_by_peers_dict["grid_button"] = "fa fa-plus-circle fa-lg fa-fw"
            #else:
            #    evaluation_by_peers_dict["grid_button"] = "fa fa-pencil fa-lg fa-fw"
            evaluation_by_peers_dict["peers_list"] = self.getPeersOrder()
            evaluation_by_peers_dict["peers_average_dict"] = self.getAveragePeer()

            jalon_bdd = self.portal_jalon_bdd
            evaluation_by_peers_dict["evaluations_notes"] = jalon_bdd.getCountEvaluationsNotes(self.getId()).first()[0]
            evaluation_by_peers_dict["verif_evaluations_notes"] = jalon_bdd.getCountVerifEvaluationsNotes(self.getId()).first()[0]
            evaluation_by_peers_dict["evaluations_validate"] = evaluation_by_peers_dict["evaluations_notes"] - evaluation_by_peers_dict["verif_evaluations_notes"]
            # LOG.info("***** evaluations_notes : %s" % evaluation_by_peers_dict["evaluations_notes"])
            # LOG.info("***** verif_evaluations_notes : %s" % evaluation_by_peers_dict["verif_evaluations_notes"])
            # LOG.info("***** evaluations_validate : %s" % evaluation_by_peers_dict["evaluations_validate"])

            evaluations_notes_informations = jalon_bdd.getInfoEvaluationsNotes(self.getId()).first()
            if evaluations_notes_informations:
                evaluation_by_peers_dict["evaluations_notes_min"] = "%.2f" % evaluations_notes_informations[0] if evaluations_notes_informations[0] else 0
                evaluation_by_peers_dict["evaluations_notes_max"] = "%.2f" % evaluations_notes_informations[1] if evaluations_notes_informations[1] else 0
                evaluation_by_peers_dict["evaluations_notes_avg"] = "%.2f" % evaluations_notes_informations[2] if evaluations_notes_informations[2] else 0

            criteria_notes_dict = {}
            criteria_notes_informations = jalon_bdd.getInfoCriteriaNotes(self.getId())
            for criteria_note in criteria_notes_informations:
                criteria_notes_dict[criteria_note[0]] = {"min": "%.2f" % criteria_note[1],
                                                         "max": "%.2f" % criteria_note[2],
                                                         "avg": "%.2f" % criteria_note[3]}
            evaluation_by_peers_dict["criteria_notes_dict"] = criteria_notes_dict
            # LOG.info("***** criteria_notes_dict : %s" % str(criteria_notes_dict))
        else:
            evaluation_by_peers_dict["macro_peers"] = "peers_student_macro"
            evaluation_by_peers_dict["table_title"] = "Dépôts à évaluer"
            evaluation_by_peers_dict["evaluate_link"] = "%s/evaluate_deposit_file_form?mode_etudiant=true" % deposit_box_link

            user_id = user.getId()
            evaluation_number = self.getNombreCorrection()
            evaluation_by_peers_dict["evaluate_button"] = False
            evaluation_by_peers_dict["corrected_evaluation_list"] = []
            evaluation_by_peers_dict["peers_correction_indication"] = "Vous n'avez aucun dépôt à évaluer."

            evaluations_note_dict = {}
            evaluation_by_peers_dict["peers_evaluations"] = self.getPeersDict(user_id)
            if len(evaluation_by_peers_dict["peers_evaluations"]):
                number = 0
                for evaluation in evaluation_by_peers_dict["peers_evaluations"]:
                    if evaluation["corrected"]:
                        number = number + 1
                        # LOG.info("***** evaluation['corrected'] : %s" % number)
                        evaluations_note_dict[number] = evaluation.get("note", "Non renseignée")
                        # LOG.info("***** evaluations_note_dict[%s] : %s" % (number, evaluations_note_dict[number]))
                evaluation_by_peers_dict["evaluations_note_dict"] = evaluations_note_dict
                # LOG.info("evaluation_by_peers_dict['evaluations_note_dict'] : %s" % evaluation_by_peers_dict["evaluations_note_dict"])
                evaluation_by_peers_dict["peers_correction_indication"] = "Vous avez évalué %i dépôts sur les %i évaluations attendues" % (number, evaluation_number)

                if number == 0:
                    evaluation_by_peers_dict["evaluate_button_name"] = "Commencer l'évaluation des dépôts"
                    evaluation_by_peers_dict["evaluate_button"] = True
                elif number < evaluation_number and not self.isFinishEvaluation():
                    evaluation_by_peers_dict["evaluate_button_name"] = "Poursuivre l'évaluation des dépôts"
                    evaluation_by_peers_dict["evaluate_button"] = True

                evaluation_by_peers_dict["corrected_evaluation_list"] = range(1, number + 1)
                # LOG.info("***** corrected_evaluation_list : %s" % evaluation_by_peers_dict["corrected_evaluation_list"])
                #jalon_bdd = self.portal_jalon_bdd
                #corrected_evaluations_dict = {}
                #corrected_evaluations = jalon_bdd.getEvaluationByCorrectedSTU(self.getId(), user_id)
                #for criteria_id in evaluation_by_peers_dict["criteria_dict"].keys():
                #    corrected_evaluations_dict[criteria_id] = {}
                #for corrected_evaluation in corrected_evaluations.all():
                #    corrected_evaluations_dict[str(corrected_evaluation[0])][corrected_evaluation[1]] = corrected_evaluation[2]
                #evaluation_by_peers_dict["corrected_evaluations_dict"] = corrected_evaluations_dict
                # LOG.info("***** corrected_evaluations_dict : %s" % corrected_evaluations_dict)
        return evaluation_by_peers_dict

    def getCriteriaDict(self, key=None):
        # LOG.info("----- getCriteriaDict -----")
        # LOG.info("***** self._crietria_dict : %s" % str(self._crietria_dict))
        if key:
            return self._crietria_dict.get(key, None)
        return dict(self._crietria_dict)

    def setCriteriaDict(self, crietria_dict):
        # LOG.info("----- setCriteriaDict -----")
        if type(self._crietria_dict).__name__ != "PersistentMapping":
            self._crietria_dict = PersistentDict(crietria_dict)
        else:
            self._crietria_dict = crietria_dict

    def getCriteriaOrder(self):
        # LOG.info("----- getCriteriaOrder -----")
        order = self._crietria_dict.keys()
        order.sort(lambda x, y: cmp(int(x), int(y)))
        # LOG.info("***** order : %s" % str(order))
        return order

    def getPeersDict(self, key=None):
        # LOG.info("----- getPeersDict -----")
        # LOG.info("***** peers_dict : %s" % str(self._peers_dict))
        if key:
            return self._peers_dict.get(key, [])
        return self._peers_dict

    def setPeersDict(self, peers_dict):
        # LOG.info("----- setPeersDict -----")
        if type(self._peers_dict).__name__ != "PersistentMapping":
            self._peers_dict = PersistentDict(peers_dict)
        else:
            self._peers_dict = peers_dict

    def getPeersOrder(self):
        # LOG.info("----- getPeersOrder -----")
        order = []
        for SESAME_ETU in self._peers_dict.keys():
            order.append({"id":  SESAME_ETU,
                          "nom": self.getNomEtudiant(SESAME_ETU)})
        order.sort(lambda x, y: cmp(x["nom"], y["nom"]))
        return order

    def getInfosPeersDict(self):
        # LOG.info("----- getInfosPeersDict -----")
        infos_peers = {}
        for SESAME_ETU in self._peers_dict.keys():
            infos_peers[SESAME_ETU] = self.getNomEtudiant(SESAME_ETU)
        return infos_peers

    def getPeerLength(self, is_personnel, user_id):
        LOG.info("----- getPeerLength -----")
        if is_personnel:
            LOG.info("is_personnel")
            LOG.info(self._peers_dict.keys())
            return len(self._peers_dict.keys())
        else:
            LOG.info(self.getPeersDict(user_id))
            return len(self.getPeersDict(user_id))

    def getDisplayPenality(self):
        # LOG.info("----- getDisplayPenality -----")
        penality = self.getPenalite()
        if penality in ["negative", "positive"]:
            return "%i %s" % (self.getAdjustementPoints(), self._penality_title[penality])
        else:
            return self._penality_title[penality]

    def getDisplayGridAccess(self):
        # LOG.info("----- getDisplayGridAccess -----")
        return "Autorisée avant évaluation" if self.getAccesGrille() else "Interdit avant évaluation"

    def getDisplayAuthorizeSelfEvaluation(self):
        # LOG.info("----- getDisplayAuthoriezSelfEvaluation -----")
        return "Auto-évaluation possible" if self.getAutoriserAutoEvaluation() else "Pas d'auto-évaluation"

    def affectDepositFile(self):
        # LOG.info("----- affectDepositFile -----")
        peers_dict = copy.deepcopy(self._peers_dict)
        # LOG.info("INTRO peers_dict : %s" % str(peers_dict))
        peers_list = peers_dict.keys()
        # LOG.info("INTRO peers_list : %s" % str(peers_list))
        correction_file_number = self.getNombreCorrection()
        # LOG.info("INTRO correction_file_number : %s" % str(correction_file_number))
        correction_file_loop = range(0, correction_file_number)
        # LOG.info("INTRO correction_file_loop : %s" % str(correction_file_loop))

        random.shuffle(peers_list)
        # LOG.info("INTRO peers_list shulle : %s" % peers_list)

        interval_list = []
        for loop in correction_file_loop:
            new_interval = random.randint(1, len(peers_list) - 1)
            while new_interval in interval_list:
                new_interval = random.randint(1, len(peers_list) - 1)
            interval_list.append(new_interval)
        # LOG.info("INTRO interval_list : %s" % str(interval_list))

        peer_index = 0
        peers_list_len = len(peers_list)
        # LOG.info("INTRO peers_list_len : %s" % str(peers_list_len))
        for peer in peers_list:
            # LOG.info("PEER peer : %s" % peer)
            for interval in interval_list:
                peer_interval = peer_index + interval
                if peer_interval >= peers_list_len:
                    peer_interval = (peer_interval - peers_list_len)
                # LOG.info("PEER peer_interval : %s" % str(peer_interval))

                affected_peer = peers_list[peer_interval]
                # LOG.info("PEER affected_peer : %s" % affected_peer)
                try:
                    peers_dict[peer].append({"peer": affected_peer, "corrected": False, "note": "Non renseignée"})
                except:
                    peers_dict[peer] = [{"peer": affected_peer, "corrected": False, "note": "Non renseignée"}]
            peer_index = peer_index + 1

        self.setPeersDict(peers_dict)
        # LOG.info("***** FINAL peers_dict : %s" % str(peers_dict))
        self.setAttributActivite({"AffectationEvaluation": True})

    def getEvaluateDepositFileForm(self, user, mode_etudiant, student_id=None, deposit_id=None):
        # LOG.info("----- getEvaluateDepositFileForm -----")
        if deposit_id == "auto":
            student_id = deposit_id = user.getId()
        is_personnel = self.isPersonnel(user, mode_etudiant)
        mode_etudiant = "false" if (not mode_etudiant) and is_personnel else mode_etudiant
        if is_personnel:
            macro_form = "teacher_evaluate"
            my_evaluate_form = self.getTeacherEvaluateDepositFileForm(student_id)
        else:
            macro_form = "student_evaluate"
            my_evaluate_form = self.getStudentEvaluateDepositFileForm(user.getId(), deposit_id)

        return {"breadcrumbs":      self.getEvaluateBreadcrumbs(is_personnel),
                "is_personnel":     is_personnel,
                "mode_etudiant":    mode_etudiant,
                "comment_dict":     {"0": "Aucun",
                                     "1": "Optionnel",
                                     "2": "Obligatoire"},
                "is_evaluable":     self.isEvaluable(),
                "macro_form":       macro_form,
                "my_evaluate_form": my_evaluate_form}

    def getStudentEvaluateDepositFileForm(self, user_id, deposit_id=None):
        # LOG.info("----- getStudentEvaluateDepositFileForm -----")
        evaluation = {}
        criteria_evaluated_dict = {}
        peers_evaluations = self.getPeersDict(user_id)

        if not deposit_id:
            deposit_index = 1
            for corrected_evaluation in peers_evaluations:
                # LOG.info("***** corrected_evaluation : %s" % str(corrected_evaluation))
                if not corrected_evaluation["corrected"]:
                    evaluation = corrected_evaluation
                    break
                deposit_index = deposit_index + 1
            deposit_name = "le dépôt n°%i" % deposit_index
        elif user_id == deposit_id:
            evaluation["peer"] = deposit_id
            deposit_name = "mon dépôt"
        else:
            deposit_name = "le dépôt n°%i" % (int(deposit_id) + 1)
            evaluation = peers_evaluations[int(deposit_id)]
            # LOG.info("***** evaluation : %s" % str(evaluation))
            criteria_evaluated_list = self.portal_jalon_bdd.getEvaluationByCorrectedAndDepositSTU(self.getId(), user_id, evaluation["peer"])
            # LOG.info("***** criteria_evaluated_list : %s" % criteria_evaluated_list.all())
            for criteria_dict in criteria_evaluated_list.all():
                criteria_evaluated_dict[criteria_dict[0]] = {"criteria_note":    criteria_dict[1],
                                                             "criteria_comment": criteria_dict[2]}
            # LOG.info("****** criteria_evaluated_dict : %s" % criteria_evaluated_dict)

        deposit_link = ""
        if evaluation:
            deposit_files = self.getFolderContents(contentFilter={"portal_type": "JalonFile", "Creator": evaluation["peer"]})
            for deposit_brain in deposit_files:
                if deposit_brain.getActif:
                    deposit_link = deposit_brain.getURL()

        return {"deposit_name":       deposit_name,
                "evaluation":         evaluation,
                "deposit_link":       "%s/at_download/file" % deposit_link,
                "criteria_dict":      self.getCriteriaDict(),
                "criteria_order":     self.getCriteriaOrder(),
                "criteria_evaluated": criteria_evaluated_dict,
                "deposit_id":         deposit_id,
                "form_link":          "%s/evaluate_deposit_file_form" % self.absolute_url()}

    def getTeacherEvaluateDepositFileForm(self, student_id):
        # LOG.info("----- getTeacherEvaluateDepositFileForm -----")
        deposit_box_id = self.getId()
        deposit_files = self.getFolderContents(contentFilter={"portal_type": "JalonFile", "Creator": student_id})
        for deposit_brain in deposit_files:
            if deposit_brain.getActif:
                deposit_link = deposit_brain.getURL()

        jalon_bdd = self.portal_jalon_bdd

        self_evaluation_dict = {}
        self_evaluation_note = ""
        has_self_evaluation = False
        request = jalon_bdd.getSelfEvaluationNote(deposit_box_id, student_id).first()
        if request:
            has_self_evaluation = True
            self_evaluation_note = request[0]

            for line in jalon_bdd.getSelfEvaluate(deposit_box_id, student_id).all():
                self_evaluation_dict[line[0]] = {"criteria_note":    line[1],
                                                 "criteria_comment": line[-1]}

        evaluation = jalon_bdd.getPeerEvaluation(deposit_box_id, student_id)
        average = jalon_bdd.getPeerAverage(deposit_box_id, student_id)

        criteria_avg = {}
        evaluation_number = self.getNombreCorrection()
        for average_dict in average.all():
            # LOG.info("***** average_dict : %s" % str(average_dict))
            criteria_error = False
            criteria_text = ""
            criteria_style = ""
            criteria_value = []
            if average_dict[2] == 2:
                criteria_error = True
                criteria_style = "color: #f04124;"
                criteria_text = "Nombre d'évaluation attendu insuffisant : %s sur %s" % (average_dict[3], evaluation_number)
            if average_dict[2] == 3:
                criteria_error = True
                criteria_style = "color: #f04124;"
                criteria_text = "Marge dépassée entre la note minimale et la note maximale"
                criteria_value = average_dict[3].split(";")
            criteria_avg[average_dict[0]] = {"criteria_avg":     average_dict[1],
                                             "criteria_value":   criteria_value,
                                             "criteria_error":   criteria_error,
                                             "criteria_style":   criteria_style,
                                             "criteria_text":    criteria_text,
                                             "criteria_note":    average_dict[1],
                                             "criteria_comment": average_dict[-1]}
        # LOG.info("***** criteria_avg : %s" % str(criteria_avg))

        criteria_eval = {}
        corrected_stu_dict = {}
        portal = self.portal_url.getPortalObject()
        for evaluation_dict in evaluation.all():
            if not evaluation_dict[1] in corrected_stu_dict:
                student = self.getIndividu(evaluation_dict[1], "dict", portal)
                if student:
                    corrected_stu_dict[evaluation_dict[1]] = "%s %s" % (student["nom"].upper(), student["prenom"].capitalize())
                else:
                    corrected_stu_dict[evaluation_dict[1]] = evaluation_dict[1]

            criteria_error = True if str(evaluation_dict[2]) in criteria_avg[evaluation_dict[0]]["criteria_value"] else False
            criteria_style = "color: #f04124;" if criteria_error else ""
            try:
                criteria_eval[evaluation_dict[0]].append({"corrected_stu":    corrected_stu_dict[evaluation_dict[1]],
                                                          "criteria_note":    evaluation_dict[2],
                                                          "criteria_comment": evaluation_dict[3],
                                                          "criteria_error":   criteria_error,
                                                          "criteria_style":   criteria_style})
            except:
                criteria_eval[evaluation_dict[0]] = [{"corrected_stu":    corrected_stu_dict[evaluation_dict[1]],
                                                      "criteria_note":    evaluation_dict[2],
                                                      "criteria_comment": evaluation_dict[3],
                                                      "criteria_error":   criteria_error,
                                                      "criteria_style":   criteria_style}]
        # LOG.info("***** criteria_eval : %s" % str(criteria_eval))

        student = self.getIndividu(student_id, "dict")
        return {"student_name":         "%s %s" % (student["nom"].upper(), student["prenom"].capitalize()),
                "deposit_link":         "%s/at_download/file" % deposit_link,
                "criteria_dict":        self.getCriteriaDict(),
                "criteria_order":       self.getCriteriaOrder(),
                "has_self_evaluation":  has_self_evaluation,
                "self_evaluation_dict": self_evaluation_dict,
                "self_evaluation_note": self_evaluation_note,
                "criteria_eval":        criteria_eval,
                "criteria_avg":         criteria_avg,
                "form_link":            "%s/evaluate_deposit_file_form" % self.absolute_url()}

    def getEvaluateBreadcrumbs(self, is_personnel=False):
        # LOG.info("----- getEvaluateBreadcrumbs -----")
        portal = self.portal_url.getPortalObject()
        parent = self.aq_parent
        breadcrumbs = [{"title": _(u"Mes cours"),
                        "icon":  "fa fa-university",
                        "link":  "%s/mes_cours" % portal.absolute_url()},
                       {"title": parent.Title(),
                        "icon":  "fa fa-book",
                        "link":  parent.absolute_url()},
                       {"title": self.Title(),
                        "icon":  "fa fa-inbox",
                        "link":  "%s?tab=peers" % self.absolute_url()}]
        if is_personnel:
            breadcrumbs.append({"title": _(u"Évaluations à vérifier"),
                                "icon":  "fa fa-list",
                                "link":  "%s/deposit_box_details_evaluations_view" % self.absolute_url()})
        return breadcrumbs

    def setEvaluatePeer(self, param_dict):
        # LOG.info("----- setEvaluatePeer -----")
        user = self.portal_membership.getAuthenticatedMember()
        if self.isPersonnel(user):
            self.setTeacherEvaluatePeer(user.getId(), param_dict)
        else:
            if param_dict["deposit_id"] == param_dict["user_id"]:
                self.setAutoEvaluatePeer(param_dict)
            else:
                self.setStudentEvaluatePeer(param_dict)

    def setStudentEvaluatePeer(self, param_dict):
        # LOG.info("----- setStudentEvaluatePeer -----")
        evaluation = {}
        evaluation_note = 0
        evaluation_coeff = 0
        peers_dict = copy.deepcopy(self.getPeersDict())
        if param_dict["deposit_id"]:
            corrected_evaluation_index = int(param_dict["deposit_id"])
            evaluation = peers_dict[param_dict["user_id"]][corrected_evaluation_index]
        else:
            corrected_evaluation_index = 0
            for corrected_evaluation in peers_dict[param_dict["user_id"]]:
                if not corrected_evaluation["corrected"]:
                    evaluation = corrected_evaluation
                    break
                corrected_evaluation_index = corrected_evaluation_index + 1

        index = 1
        jalon_bdd = self.portal_jalon_bdd
        criteria_dict = self.getCriteriaDict()
        criteria_loop = param_dict["criteria_order"].split(",")
        for loop in criteria_loop:
            criteria_id = "criteria%i" % index
            criteria_coefficient = int(criteria_dict[str(param_dict[criteria_id])]["coefficient"])
            # LOG.info("***** criteria_coefficient : %s" % criteria_coefficient)

            evaluation_note = evaluation_note + (((int(param_dict["%s-note" % criteria_id]) * 10) / int(criteria_dict[str(param_dict[criteria_id])]["notation"])) * criteria_coefficient)
            evaluation_coeff = evaluation_coeff + criteria_coefficient
            jalon_bdd.setEvaluatePeer(self.getId(), evaluation["peer"], param_dict["user_id"], param_dict[criteria_id], param_dict["%s-note" % criteria_id], param_dict["%s-comment" % criteria_id])
            evaluation["corrected"] = True
            index = index + 1

        evaluation_note_20 = "%.2f" % ((float(evaluation_note) / float(evaluation_coeff)) * 2.0)

        # LOG.info("***** evaluation : %s" % str(evaluation))
        # LOG.info("***** evaluation_note : %s" % evaluation_note)
        # LOG.info("***** evaluation_coeff : %s" % evaluation_coeff)
        # LOG.info("***** evaluation_note_20 : %s" % evaluation_note_20)
        jalon_bdd.setPeerEvaluationNote(self.getId(), evaluation["peer"], param_dict["user_id"], evaluation_note_20)
        evaluation["note"] = evaluation_note_20
        peers_dict[param_dict["user_id"]][corrected_evaluation_index] = evaluation
        self.setPeersDict(peers_dict)

    def setAutoEvaluatePeer(self, param_dict):
        # LOG.info("----- setAutoEvaluatePeer -----")
        index = 1
        evaluation_note = 0
        evaluation_coeff = 0
        jalon_bdd = self.portal_jalon_bdd
        criteria_dict = self.getCriteriaDict()
        criteria_loop = param_dict["criteria_order"].split(",")
        for loop in criteria_loop:
            criteria_id = "criteria%i" % index
            criteria_coefficient = int(criteria_dict[str(param_dict[criteria_id])]["coefficient"])
            # LOG.info("***** criteria_coefficient : %s" % criteria_coefficient)

            evaluation_note = evaluation_note + (((int(param_dict["%s-note" % criteria_id]) * 10) / int(criteria_dict[str(param_dict[criteria_id])]["notation"])) * criteria_coefficient)
            evaluation_coeff = evaluation_coeff + criteria_coefficient
            jalon_bdd.setSelfEvaluate(self.getId(), param_dict["user_id"], param_dict[criteria_id], param_dict["%s-note" % criteria_id], param_dict["%s-comment" % criteria_id])
            index = index + 1

        evaluation_note_20 = "%.2f" % ((float(evaluation_note) / float(evaluation_coeff)) * 2.0)

        # LOG.info("***** evaluation_note : %s" % evaluation_note)
        # LOG.info("***** evaluation_coeff : %s" % evaluation_coeff)
        # LOG.info("***** evaluation_note_20 : %s" % evaluation_note_20)
        jalon_bdd.setSelfEvaluationNote(self.getId(), param_dict["user_id"], evaluation_note_20)

    def setTeacherEvaluatePeer(self, user_id, param_dict):
        # LOG.info("----- setTeacherEvaluatePeer -----")
        index = 1
        evaluation_note = 0
        evaluation_coeff = 0
        jalon_bdd = self.portal_jalon_bdd
        criteria_dict = self.getCriteriaDict()
        criteria_loop = param_dict["criteria_order"].split(",")
        for loop in criteria_loop:
            criteria_id = "criteria%i" % index
            criteria_coefficient = int(criteria_dict[str(param_dict[criteria_id])]["coefficient"])
            jalon_bdd.updateAveragePeer(self.getId(), param_dict["student_id"], param_dict[criteria_id], 1, "", param_dict["%s-note" % criteria_id], param_dict["%s-note" % criteria_id], param_dict["%s-comment" % criteria_id])

            #evaluation_note = evaluation_note + (int(param_dict["%s-note" % criteria_id]) * criteria_coefficient)
            evaluation_note = evaluation_note + (((int(param_dict["%s-note" % criteria_id]) * 10) / int(criteria_dict[str(param_dict[criteria_id])]["notation"])) * criteria_coefficient)
            evaluation_coeff = evaluation_coeff + criteria_coefficient
            index = index + 1

        evaluation_note_20 = "%.2f" % ((float(evaluation_note) / float(evaluation_coeff)) * 2.0)
        jalon_bdd.updateEvaluationAverage(self.getId(), param_dict["student_id"], evaluation_note_20, False)

    def setAveragePeer(self):
        # LOG.info("----- setAveragePeer -----")
        is_verification_evaluation = {}
        jalon_bdd = self.portal_jalon_bdd
        criteria_dict = self.getCriteriaDict()
        correction_number = self.getNombreCorrection()
        jalon_bdd.deleteAverageByDepositBox(self.getId())
        jalon_bdd.deleteEvaluationsAverageByDepositBox(self.getId())
        criteria_average_list = jalon_bdd.generatePeersAverage(self.getId()).all()
        for average in criteria_average_list:
            # LOG.info("***** average : %s" % str(average))
            criteria_code = 1
            criteria_value = None
            criteria_data = criteria_dict[average[1]]
            if not average[0] in is_verification_evaluation:
                is_verification_evaluation[average[0]] = False
            if average[3] < correction_number:
                criteria_code = 2
                criteria_value = average[3]
                is_verification_evaluation[average[0]] = True
            elif average[-1] - average[-2] >= int(criteria_data["gap"]):
                criteria_code = 3
                criteria_value = "%s;%s" % (average[-1], average[-2])
                is_verification_evaluation[average[0]] = True
            # LOG.info("***** criteria_code : %s" % str(criteria_code))
            # LOG.info("***** criteria_value : %s" % str(criteria_value))
            jalon_bdd.setAveragePeer(self.getId(), average[0], average[1], criteria_code, criteria_value, "%.2f" % float(average[2]), 0, "")

        # LOG.info("***** is_verification_evaluation : %s" % is_verification_evaluation)
        evaluation_average_list = jalon_bdd.generateEvaluationsAverage(self.getId()).all()
        for average in evaluation_average_list:
            # LOG.info("***** average : %s" % str(average))
            jalon_bdd.setEvaluationAverage(self.getId(), average[0], "%.2f" % float(average[1]), is_verification_evaluation[average[0]])

    def regenerateAverage(self):
        # LOG.info("----- regenerateAverage -----")
        deposit_id = self.getId()
        jalon_bdd = self.portal_jalon_bdd
        criteria_dict = self.getCriteriaDict()
        for student in self.getPeersOrder():
            evaluation_note = 0
            evaluation_coeff = 0
            evaluation_verification = False
            for criteria_id in self.getCriteriaOrder():
                criteria_coefficient = int(criteria_dict[criteria_id]["coefficient"])
                note = jalon_bdd.getCriteriaAverage(deposit_id, student["id"], criteria_id).first()
                if note:
                    evaluation_note = evaluation_note + (((note[0] * 10) / int(criteria_dict[criteria_id]["notation"])) * criteria_coefficient)
                    evaluation_coeff = evaluation_coeff + criteria_coefficient
                    if note[1] != 1:
                        evaluation_verification = True

            evaluation_note_20 = "%.2f" % ((float(evaluation_note) / float(evaluation_coeff)) * 2.0)
            jalon_bdd.updateEvaluationAverage(deposit_id, student["id"], evaluation_note_20, evaluation_verification)

        #jalon_bdd.deleteEvaluationsAverageByDepositBox(self.getId())
        #self.setAveragePeer()

    def getAveragePeer(self):
        # LOG.info("----- getAveragePeer -----")
        average_dict = {}
        jalon_bdd = self.portal_jalon_bdd
        average_list = jalon_bdd.getPeersAverage(self.getId())
        for ligne in average_list.all():
            # LOG.info("***** ligne : %s" % str(ligne))
            criteria_note = ligne[3] if ligne[2] else ligne[4]
            try:
                average_dict[ligne[0]][ligne[1]] = criteria_note
            except:
                average_dict[ligne[0]] = {ligne[1]: criteria_note}
        return average_dict

    def isEvaluable(self):
        # LOG.info("----- isEvaluable -----")
        now = DateTime(DateTime()).strftime("%Y/%m/%d %H:%M")
        date_depot = DateTime(self.getDateDepot()).strftime("%Y/%m/%d %H:%M")

        # LOG.info("***** now : %s" % now)
        # LOG.info("***** date_depot : %s" % date_depot)
        date_correction = DateTime(self.getDateCorrection()).strftime("%Y/%m/%d %H:%M")
        # LOG.info("***** date_correction : %s" % date_correction)

        # En profil évaluation par les pairs la date de correction est obligatoire
        return False if date_correction <= now else True

    def downloadPeersResults(self, HTTP_USER_AGENT):
        # LOG.info("----- downloadPeersResults -----")
        import tempfile
        from xlwt import Workbook, XFStyle, Style, Pattern

        #translation_service = getToolByName(self, 'translation_service')
        portal_membership = getToolByName(self, "portal_membership")
        authMember = portal_membership.getAuthenticatedMember()

        fd, path = tempfile.mkstemp('.%s-xlfiletransport' % authMember.getId())
        close(fd)

        # création
        listing = Workbook(encoding="utf-8")

        # création de la feuille 1
        feuil1 = listing.add_sheet('feuille 1')

        line = 1
        column = 4
        criteria_dict = self.getCriteriaDict()
        criteria_order = self.getCriteriaOrder()
        criteria_number = len(criteria_order)

        styleEnTete = XFStyle()
        patternEnTete = Pattern()
        patternEnTete.pattern = Pattern.SOLID_PATTERN
        patternEnTete.pattern_fore_colour = Style.colour_map["black"]
        styleEnTete.pattern = patternEnTete
        styleEnTete.font.colour_index = Style.colour_map["white"]

        # ajout en-têtes tableau
        feuil1.write(0, 0, "Critère", styleEnTete)
        feuil1.write(0, 1, "Titre", styleEnTete)
        feuil1.write(0, 2, "Description", styleEnTete)

        feuil1.write(criteria_number + 2, 0, "Nom", styleEnTete)
        feuil1.write(criteria_number + 2, 1, "Prénom", styleEnTete)
        feuil1.write(criteria_number + 2, 2, "Numéro etudiant", styleEnTete)
        feuil1.write(criteria_number + 2, 3, "Identifiant", styleEnTete)

        feuil1.col(0).width = 14 * 256
        feuil1.col(1).width = 14 * 256
        feuil1.col(2).width = 14 * 256
        feuil1.col(3).width = 14 * 256

        # ajout des critères
        for criteria_id in criteria_order:
            feuil1.write(line, 0, "Critère %s" % criteria_id)
            feuil1.write(line, 1, criteria_dict[criteria_id]["title"])
            feuil1.write(line, 2, criteria_dict[criteria_id]["description"])
            # ajout en-têtes tableau
            feuil1.write(criteria_number + 2, column, "Critère %s (%s pts, coeff %s)" % (criteria_id, criteria_dict[criteria_id]["notation"], criteria_dict[criteria_id]["coefficient"]), styleEnTete)
            feuil1.col(column).width = 20 * 256
            line = line + 1
            column = column + 1

        feuil1.write(criteria_number + 2, column, "Note sur 20", styleEnTete)

        peers_list = []
        peers_id_list = []
        peers_average_dict = {}
        infos_peers = self.getInfosPeersDict()

        deposit_box_id = self.getId()
        jalon_bdd = self.portal_jalon_bdd
        for evaluation_note in jalon_bdd.getInfoEvaluationNoteByDepositStu(deposit_box_id, None).all():
            peers_average_dict[evaluation_note[0]] = {"evaluation_note":  evaluation_note[1],
                                                      "evaluation_error": evaluation_note[2]}
            peers_list.append({"id":  evaluation_note[0],
                               "nom": infos_peers[evaluation_note[0]]})
            peers_id_list.append(evaluation_note[0])

        for criteria_note in jalon_bdd.getInfoCriteriaNoteByDepositStu(deposit_box_id).all():
            if criteria_note[0] in peers_average_dict:
                peers_average_dict[criteria_note[0]][criteria_note[1]] = {"criteria_note":  criteria_note[2],
                                                                          "criteria_error": True if criteria_note[3] in [2, 3] else False}

        peers_list.sort(lambda x, y: cmp(x["nom"], y["nom"]))
        peers_infos_dict = jalon_utils.getIndividus(peers_id_list, type="dict")

        line = criteria_number + 3
        for peer in peers_list:
            column = 4
            criteria_stu = peers_average_dict[peer['id']]
            try:
                lastname = peers_infos_dict[peer['id']]["nom"]
                firstname = peers_infos_dict[peer['id']]["prenom"]
                number = peers_infos_dict[peer['id']]["num_etu"]
            except:
                lastname = peer["nom"]
                firstname = number = "Non renseigné"
            feuil1.write(line, 0, lastname)
            feuil1.write(line, 1, firstname)
            feuil1.write(line, 2, number)
            feuil1.write(line, 3, peer['id'])
            for criteria_id in criteria_order:
                criteria_note = criteria_stu[criteria_id]
                feuil1.write(line, column, criteria_note["criteria_note"])
                column = column + 1
            feuil1.write(line, column, criteria_stu["evaluation_note"])
            line = line + 1

        listing.save(path)

        fp = open(path, 'rb')
        data = fp.read()
        fp.close()
        return {"length": str(os.stat(path)[6]), "data": data}

    # #-----------------------------# #
    #  Fonctions appel à jalon_utils  #
    # #-----------------------------# #
    def test(self, condition, valeurVrai, valeurFaux):
        return jalon_utils.test(condition, valeurVrai, valeurFaux)

    def ajouterTag(self, tag):
        return jalon_utils.setTag(self, tag)

    def getTagDefaut(self):
        return jalon_utils.getTagDefaut(self)

    def isLDAP(self):
        return jalon_utils.isLDAP()

    def isAfficherElement(self, affElement, masquerElement):
        return jalon_utils.isAfficherElement(affElement, masquerElement)

    def getLocaleDate(self, date, format="%d/%m/%Y"):
        return jalon_utils.getLocaleDate(date, format)

    def jalon_quote(self, encode):
        return jalon_utils.jalon_quote(encode)

    def jalon_unquote(self, decode):
        return jalon_utils.jalon_unquote(decode)

    def retirerEspace(self, mot):
        return jalon_utils.retirerEspace(mot)

    def getShortText(self, text, limit=75):
        return jalon_utils.getShortText(text, limit)

    #   Suppression marquage HTML
    def supprimerMarquageHTML(self, chaine):
        return jalon_utils.supprimerMarquageHTML(chaine)

    #def getInfosMembre(self, etudiant_id):
    #    return jalon_utils.getInfosMembre(etudiant_id)

    def getIndividu(self, sesame, data_type=None, portal=None):
        return jalon_utils.getIndividu(sesame, data_type, portal)

registerATCT(JalonBoiteDepot, PROJECTNAME)



# -*- coding: utf-8 -*-
## Controller Python Script "import_hotpotatoes_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##
# context = context

REQUEST = context.REQUEST
form = context.REQUEST.form

context.importHotPotatoes(form["member_auth"], form["file"])

REQUEST.RESPONSE.redirect(context.absolute_url())



## Controller Python Script "download_course_students_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

request = context.REQUEST

listing = context.telechargerListingParticipants()

if listing:
    request.RESPONSE.setHeader('content-type', 'application/vnd.ms-excel')
    request.RESPONSE.setHeader('content-length', listing["length"])
    request.RESPONSE.setHeader('Content-Disposition',' attachment; filename=ListingParticipants%s.xls' % DateTime().Date())
    return listing["data"]
else:
    request.RESPONSE.redirect(context.absolute_url())


# -*- coding: utf-8 -*-
"""## Python Script "edit_wims_exercice_script"."""
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

# PloneMessageFactory permet d'utiliser le systeme i18n
from Products.CMFPlone import PloneMessageFactory as _

""" Script d'edition des exercices WIMS de Mon Espace."""
# Le contexte est un JalonExerciceWims
# context = context
REQUEST = context.REQUEST
form = REQUEST.form

object_id = context.getId()
object_link = context.absolute_url()
wims_exercice_model = form["modele"]
# attention : possible faille de securité : si un utilisateur envoi un autre member_id
user_id = context.supprimerCaractereSpeciaux(form["member_id"])
wims_exercice_folder = context.aq_parent.getMySubSpaceFolder(user_id, "Wims")

title = context.Title().decode("utf-8")
success_message       = _(u"Votre exercice « %s » a bien été modifié." % title)
unknown_model_message = _(u"Une erreur est survenue sur votre exercice « %s » (modèle introuvable ?). Merci de contacter l'administrateur de cette plateforme, en fournissant tous les détails possibles permettant de reproduire cette erreur svp." % title)
syntax_error_message  = _(u"Votre exercice « %s » n'a pas été enregistré, probablement suite à une erreur de syntaxe.<br />Par exemple, lorsque vous utilisez des parenthèses, accolades ou crochets, veillez à les placer par paires correctement fermées." % title)
unavailable_message   = _(u"Le serveur WIMS est actuellement injoignable. Merci de réessayer ultérieurement svp...")

if "save_and_test" in form:
    # url de test direct de l'exercice actuel :
    page_url = object_link
else:
    # url de modification de l'exercice
    page_url = "%s/edit_wims_exercice_form" % object_link

# Cas des Modules externes
if wims_exercice_model == "externe":
    permalink = REQUEST.get("permalink", '')
    context.setProperties({"Title":     form["title"],
                           "Modele":    wims_exercice_model,
                           "Permalink": permalink})
    context.plone_utils.addPortalMessage(success_message, 'success')

# Cas classique : on modifie  l'exercice côté WIMS
else:
    rep = context.addExoWims(object_id, form["title"], user_id, wims_exercice_model, form)
    # print rep
    if not("status" in rep):
        # La creation a planté (Cause : modele inconnu ?)
        # nb : ceci ne devrait jamais survenir en modification a priori.
        # wims_exercice_folder.manage_delObjects(ids=[object_id])
        context.plone_utils.addPortalMessage(unknown_model_message, 'error')
    else:
        if rep["status"] == "OK":
            # L'appel à WIMS s'est bien passé, on applique les modifications à l'objet Jalon
            context.plone_utils.addPortalMessage(success_message, 'success')
            context.setProperties({"Title":  form["title"],
                                   "Modele": wims_exercice_model})
        else:
            # La creation a planté coté WIMS
            if "error_code" in rep and rep["error_code"] == "503":
                # L'erreur 503 indique une interruption de WIMS
                context.plone_utils.addPortalMessage(unavailable_message, 'error')
            else:
                # on suppose que la creation a planté suite a une erreur de syntaxe.
                context.plone_utils.addPortalMessage(syntax_error_message, 'error')
            page_url = object_link

context.REQUEST.RESPONSE.redirect(page_url)



# -*- coding: utf-8 -*-

from zope.interface import implements
from zope.component import getMultiAdapter

from Products.Archetypes import public as atpublic
from Products.ATContentTypes.content.document import ATDocumentBase, ATDocumentSchema
from Products.ATContentTypes.content.base import registerATCT
from Products.CMFCore.utils import getToolByName

from jalon.content import contentMessageFactory as _
from jalon.content.config import PROJECTNAME
from jalon.content.interfaces import IJalonForum

import jalon_utils

JalonForumSchema = ATDocumentSchema.copy() + atpublic.Schema((
    atpublic.StringField("dateAff",
                         required=False,
                         accessor="getDateAff",
                         searchable=False,
                         widget=atpublic.StringWidget(label=_(u"Date d'affichage"),
                                                      description=_(u"Description de la date d'affichage"),)
                         ),
    atpublic.StringField("dateMasq",
                         required=False,
                         accessor="getDateMasq",
                         searchable=False,
                         widget=atpublic.StringWidget(label=_(u"Date à laquelle cette activité est masquée"),
                                                      description=_(u"Date servant à masquer cette activité"),)
                         ),
))


class JalonForum(ATDocumentBase):
    """ un forum pour Jalon
    """

    implements(IJalonForum)
    meta_type = 'JalonForum'
    schema = JalonForumSchema
    schema['description'].required = True
    schema['description'].widget.label = "Sujet"
    schema['text'].required = False
    schema['text'].mode = "r"

    """
    def setAttributCours(self, form):
        self.setProperties(form)
        if "Title" in form:
            self.aq_parent.modifierInfosElementPlan(self.getId(), form["Title"])
        self.reindexObject()
    """

    def setProperties(self, dico):
        keys = dico.keys()
        if "Title" in keys:
            self.aq_parent.modifierInfosElementPlan(self.getId(), dico["Title"])
        for key in keys:
            self.__getattribute__("set%s" % key)(dico[key])
        self.reindexObject()

    def isAfficherElement(self):
        return jalon_utils.isAfficherElement(self.getDateAff(), self.getDateMasq())

    def afficherRessource(self, idElement, dateAffichage, attribut):
        if attribut == "affElement":
            self.dateAff = dateAffichage
            self.dateMasq = ""
            portal = self.portal_url.getPortalObject()
            portal_workflow = getToolByName(portal, "portal_workflow")
            if portal_workflow.getInfoFor(self, "review_state", wf_id="jalon_workflow") != "pending":
                portal_workflow.doActionFor(self, "submit", "jalon_workflow")
        else:
            self.dateMasq = dateAffichage
        self.aq_parent.modifierInfosBoitePlan(idElement, {"affElement": self.getDateAff(), "masquerElement": self.getDateMasq()})
        self.reindexObject()

    def getVue(self, request):
        forum_view = getMultiAdapter((self, request), name=u'forum_view')
        return forum_view

    def test(self, condition, valeurVrai, valeurFaux):
        return jalon_utils.test(condition, valeurVrai, valeurFaux)

registerATCT(JalonForum, PROJECTNAME)



# -*- coding: utf-8 -*-

from Products.CMFCore.permissions import AccessContentsInformation

def updateCatalog(self, clear=True):
    if self.readDataFile('jalon.content-install.txt') is None:
        return

    portal = self.getSite()
    portal.setTitle("Jalon 4.5")

    if not getattr(portal.Members, "admin", None):
        portal.Members.invokeFactory(type_name='JalonFolder', id="admin")
        home = getattr(portal.Members, "admin")
        home.addSubJalonFolder("admin")

        portal.cours.invokeFactory(type_name='JalonFolder', id="admin")
        cours = getattr(portal.cours, "admin")
        cours.setTitle("Mes cours")
        cours.setPortlets()

        modMember = portal.portal_membership.getMemberById("admin")
        modMember.fullname = "Administrateur Jalon"
        modMember.email = "no-reply@jalon.unice.fr"

        logger = self.getLogger('jalon.content updateCatalog')
        logger.info('Updating catalog (with clear=%s) so items in profiles/default/structure are indexed...' % clear)
        catalog = portal.portal_catalog
        err = catalog.refreshCatalog(clear=clear)
        if not err:
            logger.info('...done.')
        else:
            logger.warn('Could not update catalog.')

    etudiants = getattr(portal, "etudiants")
    etudiants.manage_permission(AccessContentsInformation, roles=['Anonymous', 'Member', 'Authenticated', 'Manager'])



# -*- coding: utf-8 -*-
# $Id: updatelocales.py 117340 2010-05-12 09:15:25Z glenfant $
"""
i18n files maintenance utility.

Please read the README.txt beside this
file.  Portions of this code have been stolen from
PlacelessTranslationService to prevent PYTHONPATH issues
"""

import os
import sys

from i18ndude.script import rebuild_pot as i18n_rebuild_pot
from i18ndude.script import sync as i18n_sync

# ##
# # START OF CUSTOMIZABLE SECTION
# ##

# Set this to True to get verbosity and keep temporary files if any
DEBUG = True

# Your main translation domain
DOMAIN = 'jalon.content'

# Directories excluded from i18n markup search
EXCLUDED = ['profiles', 'tests']

# ##
# # END OF CUSTOMIZABLE SECTION
# ##

THIS_DIR = os.path.dirname(os.path.abspath(__file__))
ROOT = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
GENERATED_POT = '%s.pot' % DOMAIN
MANUAL_POT = '%s-manual.pot' % DOMAIN
if os.path.exists(MANUAL_POT):
    MERGE_OPT = ['--merge', MANUAL_POT]
else:
    MERGE_OPT = []
if EXCLUDED:
    EXCLUDED_OPT = '--exclude="%s"' % ' '.join(EXCLUDED)
else:
    EXCLUDED_OPT = ''
PO_FILENAME = '%s.po' % DOMAIN


def main():
    """Main function."""
    # Rebuilding the <domain>.pot
    if DEBUG:
        print "Rebuilding", GENERATED_POT
    argv = [
        'i18ndude', 'rebuild-pot',
        '--pot', GENERATED_POT,
        '--create', DOMAIN,
        MERGE_OPT,
        EXCLUDED_OPT,
        ROOT]
    sys.argv = flatten(argv)
    i18n_rebuild_pot()

    # Synching the .po files
    if DEBUG:
        print "Synching", PO_FILENAME, "files"
    argv = ['i18ndude', 'sync',
            '--pot', GENERATED_POT,
            find_po_files()
            ]
    sys.argv = flatten(argv)
    i18n_sync()

    return


def flatten(seq):
    """'applatit' une liste multiple en lite simple.

    >>> flatten([0, [1, 2, 3], [4, 5, [6, 7]]])
    [0, 1, 2, 3, 4, 5, 6, 7]
    """
    out = []
    for item in seq:
        if isinstance(item, (list, tuple)):
            out.extend(flatten(item))
        elif item:
            out.append(item)
    return out


def find_po_files():
    """List of abs paths to .po files."""
    po_files = []
    for root, dirs, files in os.walk(THIS_DIR):
        # Don' visit CSV or SVN marker dirs
        for blacklisted in ('CVS', '.svn'):
            if blacklisted in dirs:
                dirs.remove(blacklisted)
        if PO_FILENAME in files:
            po_files.append(os.path.join(root, PO_FILENAME))
    return po_files


if __name__ == '__main__':
    main()



# Controller Python Script "edit_tag"
# bind container=container
# bind context=context
# bind namespace=
# bind script=script
# bind subpath=traverse_subpath
# parameters=
# title=Modifie un tag
##

form = context.REQUEST.form
context.editTagFolder(form["tag_id"], form["title"])
context.REQUEST.RESPONSE.redirect(context.absolute_url())



## Controller Python Script "affect_deposit_file_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

context.affectDepositFile()

HTTP_X_REQUESTED_WITH = False
if context.REQUEST.HTTP_X_REQUESTED_WITH == 'XMLHttpRequest':
    HTTP_X_REQUESTED_WITH = True

if not HTTP_X_REQUESTED_WITH:
    context.REQUEST.RESPONSE.redirect("%s?tab=peers&amp;mode_etudiant=false" % context.absolute_url())
else:
    return context.absolute_url()


## Controller Python Script "order_course_map_item_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

context.orderCourseMapItems(context.REQUEST.form["plan"])



# Controller Python Script "add_deposit_box_criteria_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

param = {}
form = context.REQUEST.form

criteria_dict = dict(context.getCriteriaDict())
keys = criteria_dict.keys()
keys.sort(lambda x, y: cmp(int(x), int(y)))

index = "1"
if criteria_dict:
    index = str(int(keys[-1]) + 1)

criteria_dict[index] = {"title":       form["title"],
                        "description": form["description"],
                        "notation":    form["notation"],
                        "coefficient": form["coefficient"],
                        "gap":         form["gap"],
                        "comment":     form["comment"],
                        "comment_min": form["comment_min"],
                        "comment_max": form["comment_max"]}

context.setCriteriaDict(criteria_dict)

redirection = "%s/deposit_box_criteria_view?criteria_id=%s" % (context.absolute_url(), index)

if context.REQUEST.HTTP_X_REQUESTED_WITH != 'XMLHttpRequest':
    context.REQUEST.RESPONSE.redirect(redirection)
else:
    return redirection


# -*- coding: utf-8 -*-
"""jalon_activity.py definit la class JalonActivity."""
from zope.interface import implements

# from Products.Archetypes.public import *
from OFS.SimpleItem import SimpleItem

from jalon.content.interfaces import IJalonActivity
from jalon.content import contentMessageFactory as _

from DateTime import DateTime

# Messages de debug :
from logging import getLogger
LOG = getLogger("[JalonActivity]")


class JalonActivity(SimpleItem):
    u"""Une activité d'un cours Jalon."""

    implements(IJalonActivity)
    meta_type = 'JalonActivity'

    # -------------------------- #
    # Fonctions onglet Documents #
    # -------------------------- #
    def isChecked(self, idElement, formulaire, listeElement=None):
        """check if idElement is checked."""
        # LOG.info("----- isChecked -----")
        if formulaire == "ajout-sujets":
            if idElement in list(self.getListeSujets()):
                return 1
            return 0
        if formulaire == "ajout-corrections":
            if idElement in list(self.getListeCorrections()):
                return 1
            return 0

    def editCourseItemVisibility(self, item_id, item_date, item_property_name, is_update_from_title=False):
        u"""Modifie l'etat de la ressource quand on modifie sa visibilité."""
        # LOG.info("----- editCourseItemVisibility -----")
        is_deposit_box = True if item_id == self.getId() else False

        if is_deposit_box:
            course = self.aq_parent
            if item_property_name == "affElement":
                self.dateAff = item_date
                self.dateMasq = ""
            else:
                self.dateMasq = item_date
                course.deleteCourseActuality(item_id)
            course.editCourseItemVisibility(item_id, item_date, item_property_name, False)
            course.setCourseProperties({"DateDerniereModif": DateTime()})
        else:
            item_properties = self.getDocumentsProperties(item_id)
            if item_property_name == "affElement":
                item_properties["masquerElement"] = ""
            item_properties[item_property_name] = item_date
            self._infos_element[item_id] = item_properties
            self.setDocumentsProperties(self._infos_element)

    def addMySpaceItem(self, folder_object, item_id, item_type, user_id, display_item, map_position, display_in_plan, portal_workflow):
        """Met a jour les related Items de l'activité et de l'element de Mes ressources qu'on lui ajoute."""
        # LOG.info("----- addMySpaceItem -----")
        item_id_no_dot = item_id.replace(".", "*-*")

        item_object = getattr(folder_object, item_id)

        complement_element = None
        if item_type in ["Video", "VOD"]:
            complement_element = {"value":  display_in_plan,
                                  "auteur": item_object.getVideoauteurname(),
                                  "image":  item_object.getVideothumbnail()}

        item_object_related = item_object.getRelatedItems()
        if self not in item_object_related:
            item_object_related.append(self)
            item_object.setRelatedItems(item_object_related)
            item_object.reindexObject()

        activity_related = self.getRelatedItems()
        if item_object not in activity_related:
            activity_related.append(item_object)
            self.setRelatedItems(activity_related)

        return {"item_id_no_dot":  item_id_no_dot,
                "item_type":       item_type,
                "item_title":      item_object.Title(),
                "item_complement": complement_element,
                "item_object":     item_object}
        # self.addItemProperty(item_id_no_dot, item_type, item_object.Title(), user_id, display_item, complement_element)

    def addItemProperty(self, item_id, item_type, item_title, item_creator, display_item, complement_element):
        """Ajoute un element à la liste des sujets d'une activité."""
        # LOG.info("----- addItemProperty -----")

        items_properties = self.getDocumentsProperties()
        if item_id not in items_properties:
            items_properties[item_id] = {"titreElement":    item_title,
                                         "typeElement":     item_type,
                                         "createurElement": item_creator,
                                         "affElement":      display_item,
                                         "masquerElement":  ""}

            if complement_element:
                items_properties[item_id]["complementElement"] = complement_element
            self.setDocumentsProperties(items_properties)
            listeSujets = list(self.getListeSujets())
            listeSujets.append(item_id)
            setattr(self, "listeSujets", tuple(listeSujets))
            message = _(u"'${item_title}' a bien été ajouté aux documents enseignants.",
                        mapping={'item_title': item_title.decode("utf-8")})
            self.plone_utils.addPortalMessage(message, type='success')

    def displayDocumentsList(self, is_personnel, portal):
        """Fournit la liste des documents à afficher."""
        # LOG.info("----- displayDocumentsList -----")
        course_parent = self.aq_parent

        documents_list = []
        documents_dict = self.getDocumentsProperties()
        for document_id in self.getDocumentsList():
            document_properties = documents_dict[document_id]

            document_dict = {"document_id":      document_id,
                             "document_title":   document_properties["titreElement"],
                             "document_drop_id": "drop-%s" % document_id.replace("*-*", ""),
                             "document_link":    ""}

            is_display_item = self.isAfficherElement(document_properties["affElement"], document_properties["masquerElement"])
            document_dict["is_display_item_bool"] = True if is_display_item["val"] else False
            document_dict["is_display_item_icon"] = "fa %s fa-fw fa-lg no-pad right alert" % is_display_item["icon"]
            document_dict["is_display_item_text"] = is_display_item["legende"]

            if is_personnel or document_dict["is_display_item_bool"]:
                document_dict["document_link"] = "/".join([portal.absolute_url(), "Members", document_properties["createurElement"], course_parent._type_folder_my_space_dict[document_properties["typeElement"].replace(" ", "")], document_id.replace("*-*", "."), "view"])
                if is_personnel:
                    document_dict["document_actions"] = self.getItemActions(course_parent, document_properties, document_dict["is_display_item_bool"])
                documents_list.append(document_dict)
        return documents_list

    def getCourseItemProperties(self, key=None):
        # LOG.info("----- getCourseItemProperties -----")
        return self.getDocumentsProperties(key)

    def getItemActions(self, course_parent, item_properties, is_display_item_bool):
        """Fournit la liste des actions possibles pour un sous-element "item_properties" de l'activité."""
        # LOG.info("----- getItemActions -----")
        item_actions = course_parent._item_actions[:]

        if is_display_item_bool:
            del item_actions[0]
        else:
            del item_actions[1]

        # Retire l'option "supprimer"
        del item_actions[-1]
        # Retire l'option "jalonner"
        del item_actions[-2]
        # Retire l'option "modifier"
        del item_actions[-2]

        return item_actions

    def getDisplayItemForm(self, item_id):
        """Fournit les infos du formulaire d'affichage/masquage de l'activité."""
        # LOG.info("----- getDisplayItemForm -----")
        if self.getId() == item_id:
            # Pour l'affichage de l'activité elle-même, on fait appel à la fonction getDisplayItemForm() du cours.
            form_properties = self.aq_parent.getDisplayItemForm(item_id)
        else:
            # Pour l'affichage des elements inclus dans l'activité, on appelle getDisplayItemFormProperties() du cours.
            item_properties = self.getDocumentsProperties(item_id)
            form_properties = self.aq_parent.getDisplayItemFormProperties(item_properties)

        return form_properties

    def getDocumentsProperties(self, key=None):
        """get Properties for one or all Documents."""
        # LOG.info("----- getDocumentsProperties (%s) -----" % self.getId())
        if key:
            return self._infos_element.get(key, None)
        return self._infos_element

    def detachDocument(self, item_id):
        # LOG.info("----- detachDocument -----")
        document_properties = self.getDocumentsProperties()
        document_dict = document_properties[item_id]
        del document_properties[item_id]
        self.setDocumentsProperties(document_properties)

        documents_list = list(self.getListeSujets())
        documents_list.remove(item_id)
        self.setListeSujets(tuple(documents_list))

        item_object = getattr(getattr(getattr(self.portal_url.getPortalObject().Members, document_dict["createurElement"]), self.aq_parent._type_folder_my_space_dict[document_dict["typeElement"].replace(" ", "")]), item_id.replace("*-*", "."))
        item_relatedItems = item_object.getRelatedItems()
        if self in item_relatedItems:
            item_relatedItems.remove(self)
            item_object.setRelatedItems(item_relatedItems)
            item_object.reindexObject()

        activity_relatedItems = self.getRelatedItems()
        activity_relatedItems.remove(item_object)
        self.setRelatedItems(activity_relatedItems)

        self.reindexObject()

    def detachAllDocuments(self):
        # LOG.info("----- detachAllDocuments -----")
        document_properties = self.getDocumentsProperties()
        #document_dict = document_properties[item_id]
        #del document_properties[item_id]
        #self.setDocumentsProperties(document_properties)

        documents_list = list(self.getListeSujets())
        #documents_list.remove(item_id)
        #self.setListeSujets(tuple(documents_list))

        for item_id in documents_list:
            document_dict = document_properties[item_id]
            item_object = getattr(getattr(getattr(self.portal_url.getPortalObject().Members, document_dict["createurElement"]), self.aq_parent._type_folder_my_space_dict[document_dict["typeElement"].replace(" ", "")]), item_id.replace("*-*", "."))
            item_relatedItems = item_object.getRelatedItems()
            item_relatedItems.remove(self)
            item_object.setRelatedItems(item_relatedItems)
            item_object.reindexObject()

        #activity_relatedItems = self.getRelatedItems()
        #activity_relatedItems.remove(item_object)
        #self.setRelatedItems(activity_relatedItems)

        self.reindexObject()



# -*- coding: utf-8 -*-

from zope.interface import implements
from plone.app.blob.field import BlobField
from Products.Archetypes import public as atpublic
from Products.ATContentTypes.content.document import ATDocumentBase, ATDocumentSchema
from Products.ATContentTypes.content.base import registerATCT
from Products.CMFCore.utils import getToolByName

from jalon.content import contentMessageFactory as _
from jalon.content.config import PROJECTNAME
from jalon.content.interfaces import IJalonFile

import jalon_utils

from logging import getLogger
LOG = getLogger('[JalonFile]')

JalonFileSchema = ATDocumentSchema.copy() + atpublic.Schema((
    BlobField('file',
              widget=atpublic.FileWidget(label='A file',
                                         description='Some file'),
              required=True,
              ),
    atpublic.StringField("actif",
                         required=False,
                         accessor="getActif",
                         searchable=False,
                         default="actif",
                         widget=atpublic.StringWidget(label=_(u"Actif"),)),
    atpublic.TextField("correction",
                       required=True,
                       accessor="getCorrection",
                       searchable=False,
                       widget=atpublic.TextAreaWidget(label=_(u"Correction"),)),
    atpublic.StringField("note",
                         required=False,
                         accessor="getNote",
                         searchable=False,
                         widget=atpublic.StringWidget(label=_(u"Note ou appréciation"),)),
))


class JalonFile(ATDocumentBase):
    """ Un fichier pour une boite de dépot d'un cours Jalon
    """

    implements(IJalonFile)
    meta_type = 'JalonFile'
    schema = JalonFileSchema
    schema['description'].required = False
    schema['description'].widget.label = "Commentaire"
    schema['description'].widget.description = ""
    schema['text'].required = False
    schema['text'].mode = "r"

    def setProperties(self, dico):
        for key in dico.keys():
            self.__getattribute__("set%s" % key)(dico[key])
        self.reindexObject()

    def getAuteur(self):
        return jalon_utils.getIndividu(self.Creator(), "dict")["fullname"]

    def isNotStandard(self):
        # LOG.info("----- isNotStandard -----")
        return self.aq_parent.isNotStandard()

    def getCorrectionDepot(self):
        return str(self.correction)

    def getFichierCorrection(self):
        corrections = getattr(self.aq_parent, "corrections", None)
        if corrections:
            fichier_correction = getattr(corrections, "Correction_%s" % self.getId(), None)
            if fichier_correction:
                return "%s/at_download/file" % fichier_correction.absolute_url()
        return None

    def getSize(self):
        return self.file.get_size()

    def notifierCorrection(self, correction=False, note=False):
        LOG.info("----- notifierCorrection START -----")
        boite = self.aq_parent
        cours = boite.aq_parent
        #if not boite.getNotificationCorrection() and not boite.getNotificationNotation():
        #    LOG.info("Notification correction et notation désactivée")
        #    return None

        is_notifier_correction = True if correction and boite.getNotificationCorrection() else False
        is_notifier_note = True if note and boite.getNotificationNotation() else False

        if is_notifier_correction and is_notifier_note:
            LOG.info("Envoyer mail Correction et Note")
            self.envoyerMailCorrectionNote(boite, cours, correction, note)
            LOG.info("----- notifierCorrection END -----")
            return None

        if is_notifier_correction:
            LOG.info("Envoyer mail Correction")
            self.envoyerMailCorrectionNote(boite, cours, correction, note)
            LOG.info("----- notifierCorrection END -----")
            return None

        if is_notifier_note:
            LOG.info("Envoyer mail Note")
            self.envoyerMailCorrectionNote(boite, cours, correction, note)
            LOG.info("----- notifierCorrection END -----")
            return None

        LOG.info("Notification correction et notation désactivée")
        LOG.info("----- notifierCorrection END -----")

    def envoyerMailCorrectionNote(self, boite, cours, correction=False, note=False):
        LOG.info("----- envoyerMailCorrectionNote START -----")
        portal = getToolByName(self, "portal_url").getPortalObject()
        portal_membership = getToolByName(self, 'portal_membership')

        authMember = portal_membership.getAuthenticatedMember()
        send_from = authMember.getProperty("email")
        if not send_from:
            send_from = portal.getProperty("email_from_address")

        LOG.info(send_from)

        etudiant = portal_membership.getMemberById(self.Creator())
        send_to = etudiant.getProperty("email")
        if not send_to:
            LOG.info("Pas d'email de destinataire")
            return None

        LOG.info(send_to)

        if correction:
            objet = "Une correction est disponible"
        if note:
            objet = "Une note est disponible"
        if correction and note:
            objet = "Une correction et une note sont disponibles"

        form = {"de":    send_from,
                "a":     send_to,
                "objet": objet}

        message = ["Bonjour %s\n" % etudiant.getProperty("fullname")]
        message.append("%s pour votre dépôt \"%s\" dans :" % (objet, self.Title()))
        message.append("    - la boite \"%s\"" % boite.Title())
        message.append("    - du cours \"%s\"" % cours.Title())
        message.append("    - auteur du cours : \"%s\"" % cours.getAuteur()["fullname"])
        coAuteurs = cours.getCoAuteursCours()
        if coAuteurs:
            message.append("    - co-auteurs du cours :")
            for coAuteur in coAuteurs:
                message.append("        - %s" % coAuteur["fullname"])
        message.append("Consulter sur %s : %s.\n" % (portal.Title(), self.aq_parent.absolute_url()))
        message.append("Cordialement,")
        message.append("L'équipe %s" % portal.Title())
        form["message"] = "\n".join(message)
        LOG.info(form["message"])
        try:
            jalon_utils.envoyerMail(form)
        except:
            LOG.info("----- erreur envoi email -----")
        LOG.info("----- envoyerMailCorrectionNote END -----")

registerATCT(JalonFile, PROJECTNAME)



## Controller Python Script "edit_peers_grid_access_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Activite edit
##

#context = context
form = context.REQUEST.form

context.setProperties({"AccesGrille": form["grid_access"]})

redirection = "%s?tab=%s" % (context.absolute_url(), "peers")

if context.REQUEST.HTTP_X_REQUESTED_WITH != 'XMLHttpRequest':
    context.REQUEST.RESPONSE.redirect(redirection)
else:
    return redirection



# -*- coding: utf-8 -*-
from Products.Five.browser import BrowserView
from zope.component import getMultiAdapter

from jalon.content import contentMessageFactory as _

from logging import getLogger
LOG = getLogger('[MySpaceView]')


class MySpaceView(BrowserView):
    """Class pour le first_page
    """

    def __init__(self, context, request):
        # LOG.info("----- Init -----")
        BrowserView.__init__(self, context, request)
        self.context = context
        self.request = request

    def isAnonymous(self):
        portal_state = getMultiAdapter((self.context, self.request),
                                       name=u'plone_portal_state')
        return portal_state.anonymous()

    def getBreadcrumbs(self):
        return [{"title": _(u"Mes ressources"),
                 "icon":  "fa fa-folder-open",
                 "link":  self.context.absolute_url()}]

    def getTags(self, folder, selected_tags_list):
        # LOG.info.info("----- getTags -----")
        #tags_dict = {"last": "Les 20 derniers"}
        tags_list = []
        tags = list(folder.Subject())
        # LOG.info.info(tags)
        tags_dict = folder.getSubjectsDict()
        # LOG.info.info(tags_dict)
        #tags.sort()
        for tag in tags:
            #tags_dict[tag] = tag
            tags_list.append({"tag_id":    tag,
                              "tag_title": tags_dict[tag],
                              "tag_css":   "filter-button selected" if tag in selected_tags_list else "filter-button unselected"})

        tags_list.sort(lambda x, y: cmp(x["tag_title"], y["tag_title"]))
        tags_list.insert(0, {"tag_id":    "last",
                             "tag_title": "Les 20 derniers",
                             "tag_css":   "filter-button fixed_filter selected" if "last" in selected_tags_list else "filter-button fixed_filter unselected"})
        tags_dict["last"] = "Les 20 derniers"
        return {"tags_dict": tags_dict,
                "tags_list": tags_list}

    def getOneAndSelectedTag(self, items_list, selected_tags_list, tags_dict):
        is_no_items = False
        if not items_list:
            if "last" in selected_tags_list:
                selected_tags_list.remove("last")
            is_no_items = True if len(selected_tags_list) == 0 else False

        one_tag = ""
        is_one_tag = False
        try:
            selected_tags_list.remove("")
        except:
            pass
        if len(selected_tags_list) == 1:
            is_one_tag = True
            one_tag = {"tag_id": selected_tags_list[-1],
                       "tag_title": tags_dict[selected_tags_list[-1]]}

        is_selected_tags = "remove_tags_objects" if selected_tags_list != ["last"] and len(selected_tags_list) >= 1 else "isnt_selected_tags"

        return {"is_no_items":      is_no_items,
                "is_one_tag":       is_one_tag,
                "one_tag":          one_tag,
                "is_selected_tags": is_selected_tags}

    def getItemsList(self, folder, selected_tags_list, content_filter):
        # LOG.info("----- getItemsList -----")
        content_filter["sort_on"] = "modified"
        content_filter["sort_order"] = "descending"
        if selected_tags_list and selected_tags_list != ["last"]:
            last = False
            subjects = []
            if "last" in selected_tags_list:
                selected_tags_list.remove("last")
                last = True
            for tag in selected_tags_list:
                subjects.append(tag)
            if len(subjects) > 1:
                content_filter['Subject'] = {'query': subjects, 'operator': 'and'}
            else:
                content_filter['Subject'] = subjects[0]
            if last:
                #content_filter["sort_on"] = "modified"
                #content_filter["sort_order"] = "descending"
                return folder.getFolderContents(contentFilter=content_filter, batch=True, b_size=20)
            else:
                return folder.getFolderContents(contentFilter=content_filter)
        elif selected_tags_list == ["last"]:
            #content_filter["sort_on"] = "modified"
            #content_filter["sort_order"] = "descending"
            return folder.getFolderContents(contentFilter=content_filter, batch=True, b_size=20)
        else:
            #content_filter["sort_on"] = "modified"
            #content_filter["sort_order"] = "descending"
            return folder.getFolderContents(contentFilter=content_filter)



## Controller Python Script "delete_course_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Delete objects from a folder
##

from Products.CMFPlone import PloneMessageFactory as _
# from OFS.ObjectManager import BeforeDeleteException
# context = context

req = context.REQUEST
# req.set("tab", "2")
course_id = req.form["course_id"]
course_user_folder = context.getCourseUserFolder(req.form["user_id"])

paths = ["%s/%s" % ("/".join(course_user_folder.getPhysicalPath()), course_id)]

putils = context.plone_utils

status = 'failure'
message = _(u'Please select one or more items to delete.')

# a hint to the link integrity code to indicate the number of events to
# expect, so that all integrity breaches can be handled in a single form
# only;  normally the adapter (LinkIntegrityInfo) should be used here, but
# this would make CMFPlone depend on an import from LinkIntegrity, which
# it shouldn't...
context.REQUEST.set('link_integrity_events_to_expect', len(paths))

# Dans un cours
for path in paths:
    course = getattr(course_user_folder, course_id)
    related_items_list = course.getRelatedItems()
    for related_item in related_items_list:
        links = related_item.getRelatedItems()
        try:
            links.remove(course)
            related_item.setRelatedItems(links)
            related_item.reindexObject()
        except:
            pass
    # Suppressions suplementaires (sur les serveurs tiers)
    if course.getListeClasses():
        course.supprimerActivitesWims()
    course.deleteDepositBox()
    # suppression des tabBU sur les ressources
    # course.tagBU("remove")

success, failure = putils.deleteObjectsByPaths(paths, REQUEST=req)

if success:
    status = 'success'
    # Le message "elements supprimés" est maintenant géré en ajax
    #  message = _(u'Item(s) deleted.')
    # Possible lenteur quand bcp d'objets dans le catalogue
    #  catalog = context.portal_catalog
    #  err = catalog.refreshCatalog(clear=True)


if failure:
    # we want a more descriptive message when trying
    # to delete locked item
    from Products.CMFDefault.exceptions import ResourceLockedError
    other = []
    locked = []
    message = str(failure)
    for key, value in failure.items():
        # below is a clever way to check exception type
        try:
                raise value
        except ResourceLockedError:
                locked.append(key)
        except:
                other.append(key)
        else:
                other.append(key)
    # locked contains ids of items that cannot be deleted,
    # because they are locked; other contains ids of items
    # that cannot be deleted for other reasons;
    # now we need to construct smarter error message
    msgs = []
    mapping = {}

    if locked:
        mapping[u'lockeditems'] = ', '.join(locked)
        message = _(u'These items are locked for editing: ${lockeditems}.', mapping=mapping)
    else:
        mapping[u'items'] = ', '.join(other)
        message = _(u'${items} could not be deleted.', mapping=mapping)

    context.plone_utils.addPortalMessage(message)

if context.REQUEST.HTTP_X_REQUESTED_WITH != 'XMLHttpRequest':
    context.REQUEST.RESPONSE.redirect(context.absolute_url())
else:
    return context.restrictedTraverse("mes_cours")()



# -*- coding: utf-8 -*-

from zope.component import getMultiAdapter
from Products.CMFCore.utils import getToolByName
from plone.app.layout.viewlets.common import ViewletBase

from jalon.content.content import jalon_encode


class JalonAnalytics(ViewletBase):
    """Class pour le first_page"""

    codeDeb = ["<script type='text/javascript'>"]
    codeDeb.append("var _gaq = _gaq || [];")
    codeDeb.append("_gaq.push(['_setAccount', '*id_account*']);")
    codeDeb.append("_gaq.push(['_setDomainName', '*id_domain*']);")
    codeDeb.append("_gaq.push(['_trackPageview']);")

    codeFin = [""]
    codeFin.append("(function() {")
    codeFin.append("  var ga = document.createElement('script'); ga.type = 'text/javascript'; ga.async = true;")
    codeFin.append("  ga.src = ('https:' == document.location.protocol ? 'https://ssl' : 'http://www') + '.google-analytics.com/ga.js';")
    codeFin.append("  var s = document.getElementsByTagName('script')[0]; s.parentNode.insertBefore(ga, s);")
    codeFin.append("})();")
    codeFin.append("")
    codeFin.append("</script>")

    def __init__(self, *args, **kwargs):
        super(JalonAnalytics, self).__init__(*args, **kwargs)

    def isAnonyme(self):
        portal_state = getMultiAdapter((self.context, self.request), name=u'plone_portal_state')
        return portal_state.anonymous()

    def googleAnalytics(self):
        portal_jalon_properties = getToolByName(self, "portal_jalon_properties")
        ga_properties = portal_jalon_properties.getPropertiesGA()
        if not ga_properties["activer_ga"]:
            return ""
        code = self.codeDeb[:]
        portal_state = getMultiAdapter((self.context, self.request), name=u'plone_portal_state')
        portal = portal_state.portal()
        authMember = portal.portal_membership.getAuthenticatedMember()

        code[2] = code[2].replace("*id_account*", ga_properties["ga_id_account"])
        code[3] = code[3].replace("*id_domain*", ga_properties["ga_id_domain"])

        variable = []
        if not (authMember.has_role("Anonymous") or authMember.has_role("Manager")):
            roles = authMember.getRoles()
            try:
                roles.remove("Member")
            except:
                pass
            try:
                roles.remove("Authenticated")
            except:
                pass
            variable.append("_gaq.push(['_setCustomVar',")
            variable.append("           1,")
            variable.append("           'User Type',")
            #Si roles[0] ne fonctionne pas, c'est certainement que le ldap n'est pas bien configuré dans acl_users
            variable.append("           '%s'," % roles[0])
            variable.append("           1")
            variable.append("           ]);")
            variable.append("_gaq.push(['_setCustomVar',")
            variable.append("           2,")
            variable.append("           'User Id',")
            variable.append("           '%s'," % jalon_encode.encodeTexte(ga_properties["ga_cryptage"], authMember.getId()))
            variable.append("           1")
            variable.append("           ]);")
        if self.context.Type() == "Jalon Cours":
            variable.append("_gaq.push(['_setCustomVar',")
            variable.append("           3,")
            variable.append("           'Id Cours',")
            variable.append("           '%s'," % self.context.getId())
            variable.append("           3")
            variable.append("           ]);")
        if variable:
            code.extend(variable)

        code.extend(self.codeFin)
        return "\n".join(code)



##Python Script "add_course_announce_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Ajoute une annonce au cours
##

form = context.REQUEST.form

announce_publics = tuple(form["publicsElement"]) if form.has_key("publicsElement") else ()
mail_announce = tuple(form["mailAnnonce"]) if form.has_key("mailAnnonce") else None

context.createAnnounce(form["user_id"], form["title"], form["description"], announce_publics, mail_announce)

HTTP_X_REQUESTED_WITH = True if context.REQUEST.HTTP_X_REQUESTED_WITH == 'XMLHttpRequest' else False
if not HTTP_X_REQUESTED_WITH:
    context.REQUEST.RESPONSE.redirect(context.absolute_url())
else:
    return context.absolute_url()



## Script (Python) "caslogout"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=CAS Logout
##
from Products.CMFCore.utils import getToolByName

request = container.REQUEST
portal = context.portal_url.getPortalObject()
mt = getToolByName(context, 'portal_membership')
portal_jalon_properties = getToolByName(context, 'portal_jalon_properties')

redirection = "%s?came_from=" % portal.absolute_url()
if portal_jalon_properties.getPropertiesConnexion("activer_cas"):
    membership_tool=getToolByName(context, 'portal_membership')
    member = membership_tool.getAuthenticatedMember()
    location = member.getProperty("location")
    if location != "jalon":
        cas_client_plugin = getattr(portal.acl_users, "cas_%s" % location)
        redirection = cas_client_plugin.casServerUrlPrefix + '/logout'

mt.logoutUser(REQUEST=request)

request.RESPONSE.redirect(redirection)



##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

#return context.REQUEST
context.creerCategorie(context.REQUEST.form["title"])
context.REQUEST.RESPONSE.redirect("%s/gestion_mes_cours?gestion=gestion_categorie" % context.absolute_url())


# -*- coding: utf-8 -*-
"""Controller Python Script "create_wims_exercice_group_script"."""
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Ajout et/ou modification d'un groupe d'exercices WIMS
##

from Products.CMFPlone import PloneMessageFactory as _
#context = context
form = context.REQUEST.form

wims_exercice_model = form["modele"]
user_id = context.supprimerCaractereSpeciaux(form["authMember"])
wims_exercice_folder = context.getMySubSpaceFolder(user_id, "Wims")

if "idobj" not in form:
    # Ajouter un nouveau groupe
    object_id = wims_exercice_folder.invokeFactory(type_name='JalonExerciceWims', id="%s-%s-%s" % (wims_exercice_model, user_id, DateTime().strftime("%Y%m%d%H%M%S")))
    message = "groupe_added"
else:
    # Modifier un groupe (pour plus tard...)
    object_id = form["idobj"]

# obj est un element de type "jalonexercicewims"
obj = getattr(wims_exercice_folder, object_id)

group_title = ""
if form["title"] != "":
    if "paths" in form:
        listeIdsExos = []
        for path in form["paths"]:
            if "groupe-" not in path:
                listeIdsExos.append(path.split("/")[-1])
        obj.setProperties({"Title": form["title"],
                           "Modele": wims_exercice_model,
                           "Qnum": form["qnum"],
                           "ListeIdsExos": listeIdsExos})
        rep = obj.ajouterSerie(user_id)
    else:
        rep = {"status": "ERROR", "err_code": "no_exercice_selected", "message": _(u"Aucun exercice selectionné")}
    group_title = u"« %s »" % obj.Title().decode("utf-8")
else:
    rep = {"status": "ERROR", "err_code": "no_title", "message": _(u"Le titre est obligatoire")}


if not("status" in rep):
    # La creation a planté (Cause ?)
    wims_exercice_folder.manage_delObjects(ids=[object_id])
    message = "unknown_error"
else:
    if rep["status"] != "OK":
        if "err_code" in rep:
            message = rep["message"]
        else:
            # Comment se retrouver ici ? ajouterSerie() ne fait aucun appel au serveur WIMS...
            # Ici il faudrait envoyer un mail auto à l'admin.
            message = "Une erreur inconnue est survenue. merci de de contacter l'administrateur du site svp..."
        context.plone_utils.addPortalMessage(_(u"Votre groupe d'exercices %s n'a pas été créé.<div><strong>%s</strong></div>" % (group_title, message)), 'error')
        # La creation n'a pas aboutit coté WIMS, on supprime l'objet que l'on vient de créer sur Jalon.
        wims_exercice_folder.manage_delObjects(ids=[object_id])
    else:
        context.plone_utils.addPortalMessage(_(u"Votre groupe d'exercices %s a bien été créé." % group_title), 'success')

# Tant qu'il n'y a pas de page de modification des groupes d'exos, on retourne sur la page precedente.
return context.REQUEST.RESPONSE.redirect(context.absolute_url())



##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

form = context.REQUEST.form
context.REQUEST.RESPONSE.setHeader('Content-Type', 'application/json; charset=utf-8')

if form.has_key("coauteur"):
    return context.rechercherUtilisateur(form["coauteur"], "Personnel")
else:
    return context.rechercherUtilisateur(form["groupe"], "Etudiant")



##Python Script "saveVariablesPodcast"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

REQUEST = context.REQUEST
podcasts = {"activerPodcasts": True,
            "uploadPodcasts": REQUEST.form["uploadPodcasts"],
            "dnsPodcasts":    REQUEST.form["dnsPodcasts"]}
if not "activerPodcasts" in REQUEST.form:
    podcasts["activerPodcasts"] = False

context.setVariablesPodcast(podcasts)

REQUEST.RESPONSE.redirect("%s/@@jalon-podcast?message=1" % context.absolute_url())


# -*- coding: utf-8 -*-
from Products.Five.browser import BrowserView
from zope.component import getMultiAdapter

from jalon.content import contentMessageFactory as _

from logging import getLogger
LOG = getLogger('[MyStudentsView]')


class MyStudentsView(BrowserView):
    """Class pour le first_page
    """

    def __init__(self, context, request):
        # LOG.info("----- Init -----")
        BrowserView.__init__(self, context, request)
        self.context = context
        self.request = request

    def isAnonymous(self):
        portal_state = getMultiAdapter((self.context, self.request),
                                       name=u'plone_portal_state')
        return portal_state.anonymous()

    def getBreadcrumbs(self):
        return [{"title": _(u"Mes etudiants"),
                 "icon":  "fa fa-users",
                 "link":  self.context.absolute_url()}]

    def getMyStudentsView(self, user):
        user_roles = user.getRoles()
        user_roles.remove("Authenticated")
        try:
            user_roles.remove("Member")
        except:
            pass
        user_role = user_roles[0]

        access_dict = self.getAccessDict(user.getId()) if user_role == "Personnel" else {}

        my_students_macro_dict = {"Etudiant":      "my_class_mates",
                                  "EtudiantJalon": "my_class_mates",
                                  "Personnel":     "my_students",
                                  "Secretaire":    "all_students",
                                  "Manager":       "all_students"}
        return {"my_students_macro": my_students_macro_dict[user_role],
                "access_dict":       access_dict}

    def getAccessDict(self, user_id):
        LOG.info("----- getAccessDict START-----")
        portal = self.context.portal_url.getPortalObject()
        portal_catalog = portal.portal_catalog

        LOG.info("SEARCH Creator")
        course_list = list(portal_catalog.searchResults(portal_type="JalonCours", Creator=user_id))
        LOG.info(course_list)

        LOG.info("SEARCH getAuteurPrincipal")
        course_list.extend(list(portal_catalog.searchResults(portal_type="JalonCours", getAuteurPrincipal=user_id)))
        #if author_course_list:
        #    course_list.extend(author_course_list)
        LOG.info(course_list)

        LOG.info("SEARCH getCoAuteurs")
        course_list.extend(list(portal_catalog.searchResults(portal_type="JalonCours", getCoAuteurs=user_id)))
        LOG.info(course_list)
        #coauthor_course_list = list(portal_catalog.searchResults(portal_type="JalonCours", getCoAuteurs=user_id))
        #if coauthor_course_list:
        #    course_list.extend(coauthor_course_list)

        course_access_dict = {}
        not_access_dict = {"etape":  "Le code *-* n'est plus valide pour ce diplôme.",
                           "ue":     "Le code *-* n'est plus valide pour cette UE / UEL.",
                           "uel":    "Le code *-* n'est plus valide pour cette UE / UEL.",
                           "groupe": "Le code *-* n'est plus valide pour ce groupe."}
        portal_jalon_bdd = portal.portal_jalon_bdd
        for course in course_list:
            LOG.info("FOR COURSE : %s" % course.getId)
            for course_access in course.getListeAcces:
                LOG.info("FOR COURSE ACCESS : %s" % course_access)
                access_type, access_code = course_access.split("*-*")
                LOG.info((access_type, access_code))
                if not access_code in course_access_dict:
                    access_response_request = portal_jalon_bdd.getELPData(access_code)
                    if not access_response_request:
                        access_data = [not_access_dict[access_type].replace("*-*", access_code), access_type, access_code, access_code, "0"]
                    else:
                        access_data = list(access_response_request)
                    course_access_dict[access_code] = {"access_title":       access_data[0],
                                                       "access_code":        access_data[3] if access_data[1] == "groupe" else access_data[2],
                                                       "access_type":        access_data[1],
                                                       "access_students":    access_data[4],
                                                       "access_course_list": [course.Title]}
                else:
                    if not course.Title in course_access_dict[access_code]["access_course_list"]:
                        course_access_dict[access_code]["access_course_list"].append(course.Title)
        LOG.info("----- getAccessDict END-----")
        return course_access_dict



## Controller Python Script "edit_course_property_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Cours edit
##

form = context.REQUEST.form

course_property = form["course_property"]

if course_property == "title":
    # Supp. marquage HTML éventuellement saisi.
    form["title"] = context.supprimerMarquageHTML(form["title"])

context.setCourseProperties({course_property.capitalize(): form[course_property]})
context.setCourseProperties({"DateDerniereModif" : DateTime()})

if course_property in ["acces", "description", "activer_email_forum", "add_forum_permission", "course_map_display"]:
    context.REQUEST.RESPONSE.redirect(context.absolute_url())
else:
    return context.absolute_url()



# -*- coding: utf-8 -*-
##Script (Python) "creationcompte_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Validation des formulaires
##

from Products.CMFCore.utils import getToolByName

form = context.REQUEST.form
#portal_registration = getToolByName(context, 'portal_registration')
#portal_membership = getToolByName(context, 'portal_membership')

#password = portal_registration.generatePassword()
#portal_membership.addMember(form["login"], password, ("EtudiantJalon",), "", {"fullname": form["fullname"], "email": form["email"]})
#portal_registration.registeredNotify(form["email"])

context.Members.admin.ajouterUtilisateurJalon(form)
context.REQUEST.RESPONSE.redirect("%s/confirmation_inscription" % context.absolute_url())


## Controller Python Script "edit_course_property_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Cours edit
##

form = context.REQUEST.form

context.setCoursePublicAccess(form["course_public_access"])

context.REQUEST.RESPONSE.redirect(context.absolute_url())



# -*- coding: utf-8 -*-
# Controller Python Script "add_tags_objects_script"
# bind container=container
# bind context=context
# bind namespace=
# bind script=script
# bind subpath=traverse_subpath
# parameters=
# title=
##

try:
    tags_list = tuple(context.REQUEST.form["listeTag"])
except:
    tags_list = ()

for path in context.REQUEST.form["paths"]:
    #obj = getattr(home, path.split("/")[-1])
    obj = context.restrictedTraverse(path)
    subjects = list(obj.Subject())
    for tag in tags_list:
        if not tag in subjects: subjects.append(tag)
    obj.setSubject(tuple(subjects))
    obj.reindexObject()

if context.REQUEST.HTTP_X_REQUESTED_WITH != 'XMLHttpRequest':
    context.REQUEST.RESPONSE.redirect(context.absolute_url())
else:
    return context.restrictedTraverse("mes_ressources/%s" % context.getId())()



# -*- coding: utf-8 -*-

from Crypto.Cipher import AES
import base64
import os


def encodeTexte(chemin, texte):
    # the block size for the cipher object; must be 16, 24, or 32 for AES
    BLOCK_SIZE = 32

    # the character used for padding--with a block cipher such as AES, the value
    # you encrypt must be a multiple of BLOCK_SIZE in length.  This character is
    # used to ensure that your value is always a multiple of BLOCK_SIZE
    PADDING = '{'

    # one-liner to sufficiently pad the text to be encrypted
    pad = lambda s: s + (BLOCK_SIZE - len(s) % BLOCK_SIZE) * PADDING

    # one-liners to encrypt/encode and decrypt/decode a string
    # encrypt with AES, encode with base64
    EncodeAES = lambda c, s: base64.b64encode(c.encrypt(pad(s)))

    # generate a random secret key
    try:
        file = open(chemin, "rb")
        secret = file.read()
    except IOError:
        os.open(chemin, os.O_CREAT)
        file = open(chemin, "wb")
        secret = os.urandom(BLOCK_SIZE)
        file.write(secret)
    file.close()

    # create a cipher object using the random secret
    cipher = AES.new(secret)

    # encode a string
    #print 'Encrypted string:', EncodeAES(cipher, texte)
    return EncodeAES(cipher, texte)


def decodeTexte(texte):
    # the block size for the cipher object; must be 16, 24, or 32 for AES
    BLOCK_SIZE = 32

    # the character used for padding--with a block cipher such as AES, the value
    # you encrypt must be a multiple of BLOCK_SIZE in length.  This character is
    # used to ensure that your value is always a multiple of BLOCK_SIZE
    PADDING = '{'

    # one-liners to encrypt/encode and decrypt/decode a string
    # encrypt with AES, encode with base64
    DecodeAES = lambda c, e: c.decrypt(base64.b64decode(e)).rstrip(PADDING)

    # generate a random secret key
    try:
        file = open("/Users/firos/Desktop/Formations/secretsecret", "rb")
        secret = file.read()
    except IOError:
        os.open("/Users/firos/Desktop/Formations/secret", os.O_CREAT)
        file = open("/Users/firos/Desktop/Formations/secret", "wb")
        secret = os.urandom(BLOCK_SIZE)
        file.write(secret)
    file.close()

    # create a cipher object using the random secret
    cipher = AES.new(secret)

    # decode the encoded string
    #print 'Decrypted string:', DecodeAES(cipher, encoded)
    return DecodeAES(cipher, texte)



## Controller Python Script "download_deposit_list_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

request = context.REQUEST

listing = context.telechargerListingDepots(request["HTTP_USER_AGENT"])

request.RESPONSE.setHeader('content-type', 'application/vnd.ms-excel')
request.RESPONSE.setHeader('content-length', listing["length"])
request.RESPONSE.setHeader('Content-Disposition',' attachment; filename=ListingDepot%s.xls' % DateTime().Date())

#request.RESPONSE.redirect("%s/cours_boite_view?menu=depots" % context.absolute_url())

return listing["data"]


## Controller Python Script "edit_validate_deposit_form"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

form = context.REQUEST
context.setProperties({"Actif": form["actif"]})

context.REQUEST.RESPONSE.redirect("%s?mode_etudiant=true" % context.aq_parent.absolute_url())



## Controller Python Script "delete_deposit_box_skill_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

form = context.REQUEST.form

if "competence_id" in form:
    competences = dict(context.getCompetences())
    for competence_id in form["competence_id"]:
        del competences[competence_id]

    context.setCompetences(competences)
context.REQUEST.RESPONSE.redirect("%s?tab=skills" % context.absolute_url())



import jalonfolder
import jaloncours
import jalonboitedepot
import jalonressourceexterne
import jalontermeglossaire
import jalonconnect
import jalon_utils
import jalonexercicewims
import jaloncourswims
import jalonfile
import jalonforum
import jalonannonce
import jalon_encode
import jalon_properties
import jalon_activity


## Controller Python Script "create_course_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

param = {}
REQUEST = context.REQUEST

course_user_folder = context.getCourseUserFolder(REQUEST.form["user_id"])
course_id = course_user_folder.invokeFactory(type_name='JalonCours', id="Cours-%s-%s" % (REQUEST["user_id"], DateTime().strftime("%Y%m%d%H%M%S")))

course_object = getattr(course_user_folder, course_id)
param = {
    "Title": context.supprimerMarquageHTML(REQUEST["title"]),
    "Description": REQUEST["description"]
}

course_object.setCourseProperties(param)
course_object.invokeFactory(type_name='Folder', id="annonce")
course_object.invokeFactory(type_name='Ploneboard', id="forum")

forum_folder_object = getattr(course_object, "forum")
forum_folder_object.setTitle("Liste des forums du cours")

course_object.addCourseForum("Discussion générale", "Discutez ici librement du cours.", REQUEST["user_id"])

course_object.setCourseProperties({"DateDerniereModif": DateTime()})

context.REQUEST.RESPONSE.redirect("%s?tab=2" % context.absolute_url())



## Controller Python Script "delete_object_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Delete objects from a folder
##

from Products.CMFPlone import PloneMessageFactory as _
#from OFS.ObjectManager import BeforeDeleteException

req = context.REQUEST
paths = req.get('paths', [])
nonSup = req.get("nonSup", [])

for p in paths:
    if p in nonSup:
        paths.remove(p)

putils = context.plone_utils

status = 'failure'
message = _(u'Please select one or more items to delete.')

# a hint to the link integrity code to indicate the number of events to
# expect, so that all integrity breaches can be handled in a single form
# only;  normally the adapter (LinkIntegrityInfo) should be used here, but
# this would make CMFPlone depend on an import from LinkIntegrity, which
# it shouldn't...
context.REQUEST.set('link_integrity_events_to_expect', len(paths))

#Dans un cours
listeClasses = []
if "/cours/" in context.absolute_url():
    for path in paths:
        idcours = path.split("/")[-1]
        cours = getattr(context, idcours)
        #On recupere la liste des classes WIMS du cours.
        for dico in cours.getListeClasses():
            for key in dico.keys():
                listeClasses.append(dico[key])
        items = cours.getRelatedItems()
        for item in items:
            links = item.getRelatedItems()
            try:
                links.remove(cours)
                item.setRelatedItems(links)
                item.reindexObject()
            except:
                pass
        #suppression des tabBU sur les ressources
        cours.tagBU("remove")

# On supprime les exos coté Wims avant de le faire coté jalon
if context.getId() == "Wims":
    context.delExoWims(paths)
    #Ici il faudrait analyser le retour de Wims, et ne pas continuer à supprimer coté jalon en cas d'erreur.

success, failure = putils.deleteObjectsByPaths(paths, REQUEST=req)

if success:
    status = 'success'
    #Le message "elements supprimés" est maintenant géré en ajax
    #message = _(u'Item(s) deleted.')
    # Possible lenteur quand bcp d'objets dans le catalogue
    #catalog = context.portal_catalog
    #err = catalog.refreshCatalog(clear=True)
    #Suppressions suplementaires (sur les serveurs tiers)
    if listeClasses:
        context.delClassesWims(listeClasses, context.REQUEST)
    if context.getId() in ["Webconference", "Sonorisation"]:
        context.delEnregistrementConnect(paths)

if failure:
    # we want a more descriptive message when trying
    # to delete locked item
    from Products.CMFDefault.exceptions import ResourceLockedError
    other = []
    locked = []
    message = str(failure)
    for key, value in failure.items():
        # below is a clever way to check exception type
        try:
                raise value
        except ResourceLockedError:
                locked.append(key)
        except:
                other.append(key)
        else:
                other.append(key)
    # locked contains ids of items that cannot be deleted,
    # because they are locked; other contains ids of items
    # that cannot be deleted for other reasons;
    # now we need to construct smarter error message
    msgs = []
    mapping = {}

    if locked:
        mapping[u'lockeditems'] = ', '.join(locked)
        message = _(u'These items are locked for editing: ${lockeditems}.', mapping=mapping)
    else:
        mapping[u'items'] = ', '.join(other)
        message = _(u'${items} could not be deleted.', mapping=mapping)

    context.plone_utils.addPortalMessage(message)

if context.REQUEST.HTTP_X_REQUESTED_WITH != 'XMLHttpRequest':
    context.REQUEST.RESPONSE.redirect(context.absolute_url())
else:
    return context.restrictedTraverse(context.getMySpaceFolder())()
